=== modified file 'NEWS'
--- NEWS	2005-10-17 11:41:07 +0000
+++ NEWS	2005-10-17 11:56:54 +0000
@@ -1,5 +1,44 @@
 UNRELEASED CHANGES
 
+  IMPROVEMENTS:
+
+    * The config directory is now ~/.bazaar, and there is a single file 
+      ~/.bazaar/bazaar.conf storing email, editor and other preferences.
+      (Robert Collins)
+
+    * 'bzr add' no longer takes a --verbose option, and a --quiet option
+      has been added that suppresses all output.
+
+    * Improved zsh completion support in contrib/zsh, from Clint
+      Adams.
+
+    * Builtin 'bzr annotate' command, by Martin Pool with improvements from 
+      Goffredo Baroncelli.
+    
+    * 'bzr check' now accepts -v for verbose reporting, and checks for
+      ghosts in the branch.
+
+    * New command 're-sign' which will regenerate the gpg signature for 
+      a revision.
+
+    * If you set check_signatures=require for a path in 
+      ~/.bazaar/branches.conf then bzr will invoke your
+      gpg_signing_command (defaults to gpg) and record a digital signature
+      of your commit.
+
+  TESTING:
+
+    * The 'bzr selftest --pattern' option for has been removed, now 
+      test specifiers on the command line can be simple strings, or 
+      regexps, or both.
+
+    * Passing -v to selftest will now show the time each test took to 
+      complete, which will aid in analysing performance regressions and
+      related questions.
+
+    * 'bzr selftest' runs all tests, even if one feels, unless '--one'
+      is given.
+
   INTERNALS:
 
     * New 'manifest' command and concept for making gpg-signatures 
@@ -7,6 +46,13 @@
       representation.
 
     * Config options have moved from bzrlib.osutils to bzrlib.config.
+      (Robert Collins)
+
+    * Improved command line option definitions allowing explanations
+      for individual options, among other things.  Contributed by 
+      Magnus Therning.
+
+    * Config options have moved from bzrlib.osutils to bzrlib.config.
       Configuration is now done via the config.Config interface:
       Depending on whether you have a Branch, a Location or no information
       available, construct a *Config, and use its signature_checking,
@@ -24,37 +70,6 @@
       implicitly. (Robert Collins)
 
 
-  IMPROVEMENTS:
-
-    * The config directory is now ~/.bazaar, and there is a single file 
-      ~/.bazaar/bazaar.conf storing email, editor and other preferences.
-      (Robert Collins)
-
-    * 'bzr add' no longer takes a --verbose option, and a --quiet option
-      has been added that suppresses all output.
- 
-    * 'bzr check' now accepts -v for verbose reporting, and checks for
-      ghosts in the branch.
-
-    * New command 're-sign' which will regenerate the gpg signature for 
-      a revision.
-
-    * If you set check_signatures=require for a path in 
-      ~/.bazaar/branches.conf then bzr will invoke your
-      gpg_signing_command (defaults to gpg) and record a digital signature
-      of your commit.
-      
-
-  TESTING:
-
-    * The --pattern option has been removed, now test specifiers on the
-      command line can be simple strings, or regexps, or both.
-
-    * Passing -v to selftest will now show the time each test took to 
-      complete, which will aid in analysing performance regressions and
-      related questions.
-
-
 bzr 0.1.1 2005-10-12
 
   BUG FIXES:

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2005-10-16 22:31:25 +0000
+++ bzrlib/annotate.py	2005-10-17 05:18:20 +0000
@@ -18,33 +18,55 @@
 
 # TODO: Choice of more or less verbose formats:
 # 
-# short: just show revno
-# long: revno, author, date
 # interposed: show more details between blocks of modified lines
 
 # TODO: Show which revision caused a line to merge into the parent
 
+# TODO: With --long, show entire email address, not just the first bit
+
+# TODO: perhaps abbreviate timescales depending on how recent they are
+# e.g. "3:12 Tue", "13 Oct", "Oct 2005", etc.  
+
 import sys
 import os
 import time
 
 import bzrlib.weave
-
-def annotate_file(branch, rev_id, file_id, to_file=None):
+from bzrlib.config import extract_email_address
+
+
+def annotate_file(branch, rev_id, file_id, verbose=False, full=False,
+        to_file=None):
     if to_file is None:
         to_file = sys.stdout
+
+    prevanno=''
+    for (revno_str, author, date_str, line_rev_id, text ) in \
+            _annotate_file(branch, rev_id, file_id ):
+
+        if verbose:
+            anno = '%5s %-12s %8s ' % (revno_str, author[:12], date_str)
+        else:
+            anno = "%5s %-7s " % ( revno_str, author[:7] )
+
+        if anno.lstrip() == "" and full: anno = prevanno
+        print >>to_file, '%s| %s' % (anno, text)
+        prevanno=anno
+
+def _annotate_file(branch, rev_id, file_id ):
+
     rh = branch.revision_history()
     w = branch.weave_store.get_weave(file_id, branch.get_transaction())
     last_origin = None
     for origin, text in w.annotate_iter(rev_id):
         text = text.rstrip('\r\n')
         if origin == last_origin:
-            anno = ''
+            (revno_str, author, date_str) = ('','','')
         else:
             last_origin = origin
             line_rev_id = w.idx_to_name(origin)
             if not branch.has_revision(line_rev_id):
-                anno = '???'
+                (revno_str, author, date_str) = ('?','?','?')
             else:
                 if line_rev_id in rh:
                     revno_str = str(rh.index(line_rev_id) + 1)
@@ -56,12 +78,12 @@
                                      time.gmtime(rev.timestamp + tz))
             # a lazy way to get something like the email address
             # TODO: Get real email address
-            author = line_rev_id
-            if '@' in author:
-                author = author[:author.index('@')]
-            author = author[:12]
-            anno = '%5s %-12s %8s' % (revno_str, author, date_str)
-        print '%-27.27s | %s' % (anno, text)
+            author = rev.committer
+            try:
+                author = extract_email_address(author)
+            except BzrError:
+                pass        # use the whole name
+        yield (revno_str, author, date_str, line_rev_id, text)
 
 
 if __name__ == '__main__':
@@ -76,4 +98,4 @@
     tree = b.revision_tree(b.last_revision())
     file_id = tree.inventory.path2id(rp)
     file_version = tree.inventory[file_id].revision
-    annotate_file(b, file_version, file_id, sys.stdout)
+    annotate_file(b, file_version, file_id, to_file = sys.stdout)

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-17 11:31:05 +0000
+++ bzrlib/branch.py	2005-10-17 11:56:54 +0000
@@ -35,7 +35,8 @@
                            DivergedBranches, LockError, UnlistableStore,
                            UnlistableBranch, NoSuchFile)
 from bzrlib.textui import show_status
-from bzrlib.revision import Revision
+from bzrlib.revision import Revision, is_ancestor, get_intervening_revisions
+
 from bzrlib.delta import compare_trees
 from bzrlib.tree import EmptyTree, RevisionTree
 from bzrlib.inventory import Inventory
@@ -855,51 +856,6 @@
         finally:
             self.unlock()
 
-    def common_ancestor(self, other, self_revno=None, other_revno=None):
-        """
-        >>> from bzrlib.commit import commit
-        >>> sb = ScratchBranch(files=['foo', 'foo~'])
-        >>> sb.common_ancestor(sb) == (None, None)
-        True
-        >>> commit(sb, "Committing first revision", verbose=False)
-        >>> sb.common_ancestor(sb)[0]
-        1
-        >>> clone = sb.clone()
-        >>> commit(sb, "Committing second revision", verbose=False)
-        >>> sb.common_ancestor(sb)[0]
-        2
-        >>> sb.common_ancestor(clone)[0]
-        1
-        >>> commit(clone, "Committing divergent second revision", 
-        ...               verbose=False)
-        >>> sb.common_ancestor(clone)[0]
-        1
-        >>> sb.common_ancestor(clone) == clone.common_ancestor(sb)
-        True
-        >>> sb.common_ancestor(sb) != clone.common_ancestor(clone)
-        True
-        >>> clone2 = sb.clone()
-        >>> sb.common_ancestor(clone2)[0]
-        2
-        >>> sb.common_ancestor(clone2, self_revno=1)[0]
-        1
-        >>> sb.common_ancestor(clone2, other_revno=1)[0]
-        1
-        """
-        my_history = self.revision_history()
-        other_history = other.revision_history()
-        if self_revno is None:
-            self_revno = len(my_history)
-        if other_revno is None:
-            other_revno = len(other_history)
-        indices = range(min((self_revno, other_revno)))
-        indices.reverse()
-        for r in indices:
-            if my_history[r] == other_history[r]:
-                return r+1, my_history[r]
-        return None, None
-
-
     def revno(self):
         """Return current revision number for this branch.
 
@@ -947,9 +903,6 @@
         Traceback (most recent call last):
         DivergedBranches: These branches have diverged.
         """
-        # FIXME: If the branches have diverged, but the latest
-        # revision in this branch is completely merged into the other,
-        # then we should still be able to pull.
         self_history = self.revision_history()
         self_len = len(self_history)
         other_history = other.revision_history()
@@ -969,23 +922,38 @@
 
     def update_revisions(self, other, stop_revision=None):
         """Pull in new perfect-fit revisions."""
+        # FIXME: If the branches have diverged, but the latest
+        # revision in this branch is completely merged into the other,
+        # then we should still be able to pull.
         from bzrlib.fetch import greedy_fetch
-        from bzrlib.revision import get_intervening_revisions
         if stop_revision is None:
             stop_revision = other.last_revision()
+        ### Should this be checking is_ancestor instead of revision_history?
         if (stop_revision is not None and 
             stop_revision in self.revision_history()):
             return
         greedy_fetch(to_branch=self, from_branch=other,
                      revision=stop_revision)
-        pullable_revs = self.missing_revisions(
-            other, other.revision_id_to_revno(stop_revision))
-        if pullable_revs:
-            greedy_fetch(to_branch=self,
-                         from_branch=other,
-                         revision=pullable_revs[-1])
+        pullable_revs = self.pullable_revisions(other, stop_revision)
+        if len(pullable_revs) > 0:
             self.append_revision(*pullable_revs)
 
+    def pullable_revisions(self, other, stop_revision):
+        other_revno = other.revision_id_to_revno(stop_revision)
+        try:
+            return self.missing_revisions(other, other_revno)
+        except DivergedBranches, e:
+            try:
+                pullable_revs = get_intervening_revisions(self.last_revision(),
+                                                          stop_revision, self)
+                assert self.last_revision() not in pullable_revs
+                return pullable_revs
+            except bzrlib.errors.NotAncestor:
+                if is_ancestor(self.last_revision(), stop_revision, self):
+                    return []
+                else:
+                    raise e
+        
     def commit(self, *args, **kw):
         from bzrlib.commit import Commit
         Commit().commit(self, *args, **kw)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-17 11:31:05 +0000
+++ bzrlib/builtins.py	2005-10-17 11:56:54 +0000
@@ -19,14 +19,15 @@
 import os
 
 import bzrlib
-import bzrlib.trace
-from bzrlib.trace import mutter, note, log_error, warning
+from bzrlib import BZRDIR
+from bzrlib.commands import Command
+from bzrlib.branch import Branch
 from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError, NotBranchError
 from bzrlib.errors import DivergedBranches
-from bzrlib.branch import Branch
-from bzrlib import BZRDIR
-from bzrlib.commands import Command
+from bzrlib.option import Option
 from bzrlib.revisionspec import RevisionSpec
+import bzrlib.trace
+from bzrlib.trace import mutter, note, log_error, warning
 from bzrlib.workingtree import WorkingTree
 
 
@@ -698,16 +699,22 @@
     To request a range of logs, you can use the command -r begin:end
     -r revision requests a specific revision, -r :end or -r begin: are
     also valid.
-
-    --message allows you to give a regular expression, which will be evaluated
-    so that only matching entries will be displayed.
     """
 
     # TODO: Make --revision support uuid: and hash: [future tag:] notation.
 
     takes_args = ['filename?']
-    takes_options = ['forward', 'timezone', 'verbose', 'show-ids', 'revision',
-                     'long', 'message', 'short',]
+    takes_options = [Option('forward', 
+                            help='show from oldest to newest'),
+                     'timezone', 'verbose', 
+                     'show-ids', 'revision',
+                     Option('line', help='format with one line per revision'),
+                     'long', 
+                     Option('message',
+                            help='show revisions whose message matches this regexp',
+                            type=str),
+                     Option('short', help='use moderately short format'),
+                     ]
     
     def run(self, filename=None, timezone='original',
             verbose=False,
@@ -716,10 +723,12 @@
             revision=None,
             message=None,
             long=False,
-            short=False):
+            short=False,
+            line=False):
         from bzrlib.log import log_formatter, show_log
         import codecs
-
+        assert message is None or isinstance(message, basestring), \
+            "invalid message argument %r" % message
         direction = (forward and 'forward') or 'reverse'
         
         if filename:
@@ -756,10 +765,11 @@
         # in e.g. the default C locale.
         outf = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
 
-        if not short:
-            log_format = 'long'
-        else:
+        log_format = 'long'
+        if short:
             log_format = 'short'
+        if line:
+            log_format = 'line'
         lf = log_formatter(log_format,
                            show_ids=show_ids,
                            to_file=outf,
@@ -1001,7 +1011,13 @@
     # XXX: verbose currently does nothing
 
     takes_args = ['selected*']
-    takes_options = ['message', 'file', 'verbose', 'unchanged']
+    takes_options = ['message', 'verbose', 
+                     Option('unchanged',
+                            help='commit even if nothing has changed'),
+                     Option('file', type=str, 
+                            argname='msgfile',
+                            help='file containing commit message'),
+                     ]
     aliases = ['ci', 'checkin']
 
     def run(self, message=None, file=None, verbose=True, selected_list=None,
@@ -1015,7 +1031,6 @@
         tree = WorkingTree(b.base, b)
         if selected_list:
             selected_list = [tree.relpath(s) for s in selected_list]
-            
         if message is None and not file:
             catcher = StringIO()
             show_status(b, specific_files=selected_list,
@@ -1125,12 +1140,16 @@
     fail.
     
     If arguments are given, they are regular expressions that say
-    which tests should run."""
+    which tests should run.
+    """
     # TODO: --list should give a list of all available tests
     hidden = True
     takes_args = ['testspecs*']
-    takes_options = ['verbose']
-    def run(self, testspecs_list=None, verbose=False):
+    takes_options = ['verbose', 
+                     Option('one', help='stop when one test fails'),
+                    ]
+
+    def run(self, testspecs_list=None, verbose=False, one=False):
         import bzrlib.ui
         from bzrlib.selftest import selftest
         # we don't want progress meters from the tests to go to the
@@ -1145,7 +1164,8 @@
             else:
                 pattern = ".*"
             result = selftest(verbose=verbose, 
-                              pattern=pattern)
+                              pattern=pattern,
+                              stop_on_failure=one)
             if result:
                 bzrlib.trace.info('tests passed')
             else:
@@ -1455,16 +1475,23 @@
 class cmd_annotate(Command):
     """Show the origin of each line in a file.
 
-    This prints out the given file with an annotation on the 
-    left side indicating which revision, author and date introduced the 
-    change.
+    This prints out the given file with an annotation on the left side
+    indicating which revision, author and date introduced the change.
+
+    If the origin is the same for a run of consecutive lines, it is 
+    shown only at the top, unless the --all option is given.
     """
     # TODO: annotate directories; showing when each file was last changed
     # TODO: annotate a previous version of a file
+    # TODO: if the working copy is modified, show annotations on that 
+    #       with new uncommitted lines marked
     aliases = ['blame', 'praise']
     takes_args = ['filename']
+    takes_options = [Option('all', help='show annotations on all lines'),
+                     Option('long', help='show date in annotations'),
+                     ]
 
-    def run(self, filename):
+    def run(self, filename, all=False, long=False):
         from bzrlib.annotate import annotate_file
         b = Branch.open_containing(filename)
         b.lock_read()
@@ -1474,7 +1501,7 @@
             tree = b.revision_tree(b.last_revision())
             file_id = tree.inventory.path2id(rp)
             file_version = tree.inventory[file_id].revision
-            annotate_file(b, file_version, file_id, sys.stdout)
+            annotate_file(b, file_version, file_id, long, all, sys.stdout)
         finally:
             b.unlock()
 

=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-10-12 07:28:41 +0000
+++ bzrlib/changeset.py	2005-10-14 04:53:26 +0000
@@ -17,9 +17,12 @@
 import errno
 import patch
 import stat
+from tempfile import mkdtemp
+from shutil import rmtree
 from bzrlib.trace import mutter
-from bzrlib.osutils import rename
+from bzrlib.osutils import rename, sha_file
 import bzrlib
+from itertools import izip
 
 # XXX: mbp: I'm not totally convinced that we should handle conflicts
 # as part of changeset application, rather than only in the merge
@@ -143,6 +146,9 @@
         """
         self.target = contents
 
+    def __repr__(self):
+        return "SymlinkCreate(%s)" % self.target
+
     def __call__(self, filename, conflict_handler, reverse):
         """Creates or destroys the symlink.
 
@@ -230,6 +236,72 @@
 
                     
 
+class TreeFileCreate(object):
+    """Create or delete a file (for use with ReplaceContents)"""
+    def __init__(self, tree, file_id):
+        """Constructor
+
+        :param contents: The contents of the file to write
+        :type contents: str
+        """
+        self.tree = tree
+        self.file_id = file_id
+
+    def __repr__(self):
+        return "TreeFileCreate(%s)" % self.file_id
+
+    def __eq__(self, other):
+        if not isinstance(other, TreeFileCreate):
+            return False
+        return self.tree.get_file_sha1(self.file_id) == \
+            other.tree.get_file_sha1(other.file_id)
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def write_file(self, filename):
+        outfile = file(filename, "wb")
+        for line in self.tree.get_file(self.file_id):
+            outfile.write(line)
+
+    def same_text(self, filename):
+        in_file = file(filename, "rb")
+        return sha_file(in_file) == self.tree.get_file_sha1(self.file_id)
+
+    def __call__(self, filename, conflict_handler, reverse):
+        """Create or delete a file
+
+        :param filename: The name of the file to create
+        :type filename: str
+        :param reverse: Delete the file instead of creating it
+        :type reverse: bool
+        """
+        if not reverse:
+            try:
+                self.write_file(filename)
+            except IOError, e:
+                if e.errno == errno.ENOENT:
+                    if conflict_handler.missing_parent(filename)=="continue":
+                        self.write_file(filename)
+                else:
+                    raise
+
+        else:
+            try:
+                if not self.same_text(filename):
+                    direction = conflict_handler.wrong_old_contents(filename,
+                        self.tree.get_file(self.file_id).read())
+                    if  direction != "continue":
+                        return
+                os.unlink(filename)
+            except IOError, e:
+                if e.errno != errno.ENOENT:
+                    raise
+                if conflict_handler.missing_for_rm(filename, undo) == "skip":
+                    return
+
+                    
+
 def reversed(sequence):
     max = len(sequence) - 1
     for i in range(len(sequence)):
@@ -302,6 +374,12 @@
             if mode is not None:
                 os.chmod(filename, mode)
 
+    def is_creation(self):
+        return self.new_contents is not None and self.old_contents is None
+
+    def is_deletion(self):
+        return self.old_contents is not None and self.new_contents is None
+
 class ApplySequence(object):
     def __init__(self, changes=None):
         self.changes = []
@@ -338,6 +416,12 @@
         self.base = base
         self.other = other
 
+    def is_creation(self):
+        return False
+
+    def is_deletion(self):
+        return False
+
     def __eq__(self, other):
         if not isinstance(other, Diff3Merge):
             return False
@@ -347,32 +431,44 @@
     def __ne__(self, other):
         return not (self == other)
 
+    def dump_file(self, temp_dir, name, tree):
+        out_path = os.path.join(temp_dir, name)
+        out_file = file(out_path, "wb")
+        in_file = tree.get_file(self.file_id)
+        for line in in_file:
+            out_file.write(line)
+        return out_path
+
     def apply(self, filename, conflict_handler, reverse=False):
-        new_file = filename+".new"
-        base_file = self.base.readonly_path(self.file_id)
-        other_file = self.other.readonly_path(self.file_id)
-        if not reverse:
-            base = base_file
-            other = other_file
-        else:
-            base = other_file
-            other = base_file
-        status = patch.diff3(new_file, filename, base, other)
-        if status == 0:
-            os.chmod(new_file, os.stat(filename).st_mode)
-            rename(new_file, filename)
-            return
-        else:
-            assert(status == 1)
-            def get_lines(filename):
-                my_file = file(filename, "rb")
-                lines = my_file.readlines()
-                my_file.close()
-                return lines
-            base_lines = get_lines(base)
-            other_lines = get_lines(other)
-            conflict_handler.merge_conflict(new_file, filename, base_lines, 
-                                            other_lines)
+        temp_dir = mkdtemp(prefix="bzr-")
+        try:
+            new_file = filename+".new"
+            base_file = self.dump_file(temp_dir, "base", self.base)
+            other_file = self.dump_file(temp_dir, "other", self.other)
+            if not reverse:
+                base = base_file
+                other = other_file
+            else:
+                base = other_file
+                other = base_file
+            status = patch.diff3(new_file, filename, base, other)
+            if status == 0:
+                os.chmod(new_file, os.stat(filename).st_mode)
+                rename(new_file, filename)
+                return
+            else:
+                assert(status == 1)
+                def get_lines(filename):
+                    my_file = file(filename, "rb")
+                    lines = my_file.readlines()
+                    my_file.close()
+                    return lines
+                base_lines = get_lines(base)
+                other_lines = get_lines(other)
+                conflict_handler.merge_conflict(new_file, filename, base_lines, 
+                                                other_lines)
+        finally:
+            rmtree(temp_dir)
 
 
 def CreateDir():
@@ -411,7 +507,7 @@
     """
     return ReplaceContents(FileCreate(contents), None)
 
-def ReplaceFileContents(old_contents, new_contents):
+def ReplaceFileContents(old_tree, new_tree, file_id):
     """Convenience fucntion to replace the contents of a file.
     
     :param old_contents: The contents of the file to replace 
@@ -421,7 +517,8 @@
     :return: A ReplaceContents that will replace the contents of a file a file 
     :rtype: `ReplaceContents`
     """
-    return ReplaceContents(FileCreate(old_contents), FileCreate(new_contents))
+    return ReplaceContents(TreeFileCreate(old_tree, file_id), 
+                           TreeFileCreate(new_tree, file_id))
 
 def CreateSymlink(target):
     """Convenience fucntion to create a symlink.
@@ -590,8 +687,7 @@
         :param reverse: if true, the changeset is being applied in reverse
         :rtype: bool
         """
-        return ((self.new_parent is None and not reverse) or 
-                (self.parent is None and reverse))
+        return self.is_creation(not reverse)
 
     def is_creation(self, reverse):
         """Return true if applying the entry would create a file/directory.
@@ -599,8 +695,12 @@
         :param reverse: if true, the changeset is being applied in reverse
         :rtype: bool
         """
-        return ((self.parent is None and not reverse) or 
-                (self.new_parent is None and reverse))
+        if self.contents_change is None:
+            return False
+        if reverse:
+            return self.contents_change.is_deletion()
+        else:
+            return self.contents_change.is_creation()
 
     def is_creation_or_deletion(self):
         """Return true if applying the entry would create or delete a 
@@ -608,7 +708,7 @@
 
         :rtype: bool
         """
-        return self.parent is None or self.new_parent is None
+        return self.is_creation(False) or self.is_deletion(False)
 
     def get_cset_path(self, mod=False):
         """Determine the path of the entry according to the changeset.
@@ -786,7 +886,7 @@
     :rtype: (List, List)
     """
     source_entries = [x for x in changeset.entries.itervalues() 
-                      if x.needs_rename()]
+                      if x.needs_rename() or x.is_creation_or_deletion()]
     # these are done from longest path to shortest, to avoid deleting a
     # parent before its children are deleted/renamed 
     def longest_to_shortest(entry):
@@ -833,7 +933,7 @@
             entry.apply(path, conflict_handler, reverse)
             temp_name[entry.id] = None
 
-        else:
+        elif entry.needs_rename():
             to_name = os.path.join(temp_dir, str(i))
             src_path = inventory.get(entry.id)
             if src_path is not None:
@@ -844,7 +944,8 @@
                 except OSError, e:
                     if e.errno != errno.ENOENT:
                         raise
-                    if conflict_handler.missing_for_rename(src_path) == "skip":
+                    if conflict_handler.missing_for_rename(src_path, to_name) \
+                        == "skip":
                         continue
 
     return temp_name
@@ -878,7 +979,7 @@
         if entry.is_creation(reverse):
             entry.apply(new_path, conflict_handler, reverse)
             changed_inventory[entry.id] = new_tree_path
-        else:
+        elif entry.needs_rename():
             if old_path is None:
                 continue
             try:
@@ -971,8 +1072,8 @@
 
 
 class MissingForRename(Exception):
-    def __init__(self, filename):
-        msg = "Attempt to move missing path %s" % (filename)
+    def __init__(self, filename, to_path):
+        msg = "Attempt to move missing path %s to %s" % (filename, to_path)
         Exception.__init__(self, msg)
         self.filename = filename
 
@@ -981,6 +1082,11 @@
         msg = "Conflicting contents for new file %s" % (filename)
         Exception.__init__(self, msg)
 
+class ThreewayContentsConflict(Exception):
+    def __init__(self, filename):
+        msg = "Conflicting contents for file %s" % (filename)
+        Exception.__init__(self, msg)
+
 
 class MissingForMerge(Exception):
     def __init__(self, filename):
@@ -1043,8 +1149,8 @@
     def missing_for_rm(self, filename, change):
         raise MissingForRm(filename)
 
-    def missing_for_rename(self, filename):
-        raise MissingForRename(filename)
+    def missing_for_rename(self, filename, to_path):
+        raise MissingForRename(filename, to_path)
 
     def missing_for_merge(self, file_id, other_path):
         raise MissingForMerge(other_path)
@@ -1052,6 +1158,10 @@
     def new_contents_conflict(self, filename, other_contents):
         raise NewContentsConflict(filename)
 
+    def threeway_contents_conflict(self, filename, this_contents,
+                                   base_contents, other_contents):
+        raise ThreewayContentsConflict(filename)
+
     def finalize(self):
         pass
 
@@ -1088,7 +1198,7 @@
     
     #apply changes that don't affect filenames
     for entry in changeset.entries.itervalues():
-        if not entry.is_creation_or_deletion():
+        if not entry.is_creation_or_deletion() and not entry.is_boring():
             path = os.path.join(dir, inventory[entry.id])
             entry.apply(path, conflict_handler, reverse)
 
@@ -1113,7 +1223,7 @@
     r_inventory = {}
     for entry in tree.source_inventory().itervalues():
         inventory[entry.id] = entry.path
-    new_inventory = apply_changeset(cset, r_inventory, tree.root,
+    new_inventory = apply_changeset(cset, r_inventory, tree.basedir,
                                     reverse=reverse)
     new_entries, remove_entries = \
         get_inventory_change(inventory, new_inventory, cset, reverse)
@@ -1267,12 +1377,13 @@
             return False
     return True
 
-class UnsuppportedFiletype(Exception):
-    def __init__(self, full_path, stat_result):
-        msg = "The file \"%s\" is not a supported filetype." % full_path
+class UnsupportedFiletype(Exception):
+    def __init__(self, kind, full_path):
+        msg = "The file \"%s\" is a %s, which is not a supported filetype." \
+            % (full_path, kind)
         Exception.__init__(self, msg)
         self.full_path = full_path
-        self.stat_result = stat_result
+        self.kind = kind
 
 def generate_changeset(tree_a, tree_b, interesting_ids=None):
     return ChangesetGenerator(tree_a, tree_b, interesting_ids)()
@@ -1319,7 +1430,7 @@
     def get_entry(self, file_id, tree):
         if not tree.has_or_had_id(file_id):
             return None
-        return tree.tree.inventory[file_id]
+        return tree.inventory[file_id]
 
     def get_entry_parent(self, entry):
         if entry is None:
@@ -1376,12 +1487,7 @@
         if cs_entry is None:
             return None
 
-        full_path_a = self.tree_a.readonly_path(id)
-        full_path_b = self.tree_b.readonly_path(id)
-        stat_a = self.lstat(full_path_a)
-        stat_b = self.lstat(full_path_b)
-
-        cs_entry.metadata_change = self.make_exec_flag_change(stat_a, stat_b)
+        cs_entry.metadata_change = self.make_exec_flag_change(id)
 
         if id in self.tree_a and id in self.tree_b:
             a_sha1 = self.tree_a.get_file_sha1(id)
@@ -1389,65 +1495,46 @@
             if None not in (a_sha1, b_sha1) and a_sha1 == b_sha1:
                 return cs_entry
 
-        cs_entry.contents_change = self.make_contents_change(full_path_a,
-                                                             stat_a, 
-                                                             full_path_b, 
-                                                             stat_b)
+        cs_entry.contents_change = self.make_contents_change(id)
         return cs_entry
 
-    def make_exec_flag_change(self, stat_a, stat_b):
+    def make_exec_flag_change(self, file_id):
         exec_flag_a = exec_flag_b = None
-        if stat_a is not None and not stat.S_ISLNK(stat_a.st_mode):
-            exec_flag_a = bool(stat_a.st_mode & 0111)
-        if stat_b is not None and not stat.S_ISLNK(stat_b.st_mode):
-            exec_flag_b = bool(stat_b.st_mode & 0111)
+        if file_id in self.tree_a and self.tree_a.kind(file_id) == "file":
+            exec_flag_a = self.tree_a.is_executable(file_id)
+
+        if file_id in self.tree_b and self.tree_b.kind(file_id) == "file":
+            exec_flag_b = self.tree_b.is_executable(file_id)
+
         if exec_flag_a == exec_flag_b:
             return None
         return ChangeExecFlag(exec_flag_a, exec_flag_b)
 
-    def make_contents_change(self, full_path_a, stat_a, full_path_b, stat_b):
-        if stat_a is None and stat_b is None:
-            return None
-        if None not in (stat_a, stat_b) and stat.S_ISDIR(stat_a.st_mode) and\
-            stat.S_ISDIR(stat_b.st_mode):
-            return None
-        if None not in (stat_a, stat_b) and stat.S_ISREG(stat_a.st_mode) and\
-            stat.S_ISREG(stat_b.st_mode):
-            if stat_a.st_ino == stat_b.st_ino and \
-                stat_a.st_dev == stat_b.st_dev:
-                return None
-
-        a_contents = self.get_contents(stat_a, full_path_a)
-        b_contents = self.get_contents(stat_b, full_path_b)
+    def make_contents_change(self, file_id):
+        a_contents = get_contents(self.tree_a, file_id)
+        b_contents = get_contents(self.tree_b, file_id)
         if a_contents == b_contents:
             return None
         return ReplaceContents(a_contents, b_contents)
 
-    def get_contents(self, stat_result, full_path):
-        if stat_result is None:
-            return None
-        elif stat.S_ISREG(stat_result.st_mode):
-            return FileCreate(file(full_path, "rb").read())
-        elif stat.S_ISDIR(stat_result.st_mode):
-            return dir_create
-        elif stat.S_ISLNK(stat_result.st_mode):
-            return SymlinkCreate(os.readlink(full_path))
-        else:
-            raise UnsupportedFiletype(full_path, stat_result)
 
-    def lstat(self, full_path):
-        stat_result = None
-        if full_path is not None:
-            try:
-                stat_result = os.lstat(full_path)
-            except OSError, e:
-                if e.errno != errno.ENOENT:
-                    raise
-        return stat_result
+def get_contents(tree, file_id):
+    """Return the appropriate contents to create a copy of file_id from tree"""
+    if file_id not in tree:
+        return None
+    kind = tree.kind(file_id)
+    if kind == "file":
+        return TreeFileCreate(tree, file_id)
+    elif kind in ("directory", "root_directory"):
+        return dir_create
+    elif kind == "symlink":
+        return SymlinkCreate(tree.get_symlink_target(file_id))
+    else:
+        raise UnsupportedFiletype(kind, tree.id2path(file_id))
 
 
 def full_path(entry, tree):
-    return os.path.join(tree.root, entry.path)
+    return os.path.join(tree.basedir, entry.path)
 
 def new_delete_entry(entry, tree, inventory, delete):
     if entry.path == "":

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-10-11 22:57:45 +0000
+++ bzrlib/commands.py	2005-10-17 05:18:20 +0000
@@ -38,6 +38,7 @@
 from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError, NotBranchError
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib import BZRDIR
+from bzrlib.option import Option
 
 plugin_cmds = {}
 
@@ -67,85 +68,6 @@
     return cmd[4:].replace('_','-')
 
 
-def _parse_revision_str(revstr):
-    """This handles a revision string -> revno.
-
-    This always returns a list.  The list will have one element for
-    each revision specifier supplied.
-
-    >>> _parse_revision_str('234')
-    [<RevisionSpec_int 234>]
-    >>> _parse_revision_str('234..567')
-    [<RevisionSpec_int 234>, <RevisionSpec_int 567>]
-    >>> _parse_revision_str('..')
-    [<RevisionSpec None>, <RevisionSpec None>]
-    >>> _parse_revision_str('..234')
-    [<RevisionSpec None>, <RevisionSpec_int 234>]
-    >>> _parse_revision_str('234..')
-    [<RevisionSpec_int 234>, <RevisionSpec None>]
-    >>> _parse_revision_str('234..456..789') # Maybe this should be an error
-    [<RevisionSpec_int 234>, <RevisionSpec_int 456>, <RevisionSpec_int 789>]
-    >>> _parse_revision_str('234....789') #Error ?
-    [<RevisionSpec_int 234>, <RevisionSpec None>, <RevisionSpec_int 789>]
-    >>> _parse_revision_str('revid:test@other.com-234234')
-    [<RevisionSpec_revid revid:test@other.com-234234>]
-    >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
-    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
-    >>> _parse_revision_str('revid:test@other.com-234234..23')
-    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_int 23>]
-    >>> _parse_revision_str('date:2005-04-12')
-    [<RevisionSpec_date date:2005-04-12>]
-    >>> _parse_revision_str('date:2005-04-12 12:24:33')
-    [<RevisionSpec_date date:2005-04-12 12:24:33>]
-    >>> _parse_revision_str('date:2005-04-12T12:24:33')
-    [<RevisionSpec_date date:2005-04-12T12:24:33>]
-    >>> _parse_revision_str('date:2005-04-12,12:24:33')
-    [<RevisionSpec_date date:2005-04-12,12:24:33>]
-    >>> _parse_revision_str('-5..23')
-    [<RevisionSpec_int -5>, <RevisionSpec_int 23>]
-    >>> _parse_revision_str('-5')
-    [<RevisionSpec_int -5>]
-    >>> _parse_revision_str('123a')
-    Traceback (most recent call last):
-      ...
-    BzrError: No namespace registered for string: '123a'
-    >>> _parse_revision_str('abc')
-    Traceback (most recent call last):
-      ...
-    BzrError: No namespace registered for string: 'abc'
-    >>> _parse_revision_str('branch:../branch2')
-    [<RevisionSpec_branch branch:../branch2>]
-    """
-    import re
-    old_format_re = re.compile('\d*:\d*')
-    m = old_format_re.match(revstr)
-    revs = []
-    if m:
-        warning('Colon separator for revision numbers is deprecated.'
-                ' Use .. instead')
-        for rev in revstr.split(':'):
-            if rev:
-                revs.append(RevisionSpec(int(rev)))
-            else:
-                revs.append(RevisionSpec(None))
-    else:
-        next_prefix = None
-        for x in revstr.split('..'):
-            if not x:
-                revs.append(RevisionSpec(None))
-            elif x[-1] == ':':
-                # looks like a namespace:.. has happened
-                next_prefix = x + '..'
-            else:
-                if next_prefix is not None:
-                    x = next_prefix + x
-                revs.append(RevisionSpec(x))
-                next_prefix = None
-        if next_prefix is not None:
-            revs.append(RevisionSpec(next_prefix))
-    return revs
-
-
 def _builtin_commands():
     import bzrlib.builtins
     r = {}
@@ -236,14 +158,15 @@
         repeated, etc.
 
     takes_options
-        List of options that may be given for this command.
+        List of options that may be given for this command.  These can
+        be either strings, referring to globally-defined options,
+        or option objects.  Retrieve through options().
 
     hidden
         If true, this command isn't advertised.  This is typically
         for commands intended for expert users.
     """
     aliases = []
-    
     takes_args = []
     takes_options = []
 
@@ -254,23 +177,31 @@
         if self.__doc__ == Command.__doc__:
             warn("No help message set for %r" % self)
 
+    def options(self):
+        """Return dict of valid options for this command.
+
+        Maps from long option name to option object."""
+        r = dict()
+        r['help'] = Option.OPTIONS['help']
+        for o in self.takes_options:
+            if not isinstance(o, Option):
+                o = Option.OPTIONS[o]
+            r[o.name] = o
+        return r
 
     def run_argv(self, argv):
         """Parse command line and run."""
-        args, opts = parse_args(argv)
-
+        args, opts = parse_args(self, argv)
         if 'help' in opts:  # e.g. bzr add --help
             from bzrlib.help import help_on_command
             help_on_command(self.name())
             return 0
-
-        # check options are reasonable
-        allowed = self.takes_options
+        # XXX: This should be handled by the parser
+        allowed_names = self.options().keys()
         for oname in opts:
-            if oname not in allowed:
+            if oname not in allowed_names:
                 raise BzrCommandError("option '--%s' is not allowed for command %r"
                                       % (oname, self.name()))
-
         # mix arguments and options into one dictionary
         cmdargs = _match_argform(self.name(), self.takes_args, args)
         cmdopts = {}
@@ -337,120 +268,64 @@
         parsed = [spec, None]
     return parsed
 
-
-# list of all available options; the rhs can be either None for an
-# option that takes no argument, or a constructor function that checks
-# the type.
-OPTIONS = {
-    'all':                    None,
-    'basis':                  str,
-    'diff-options':           str,
-    'help':                   None,
-    'file':                   unicode,
-    'force':                  None,
-    'format':                 unicode,
-    'forward':                None,
-    'quiet':                  None,
-    'message':                unicode,
-    'no-recurse':             None,
-    'profile':                None,
-    'revision':               _parse_revision_str,
-    'short':                  None,
-    'show-ids':               None,
-    'timezone':               str,
-    'verbose':                None,
-    'version':                None,
-    'email':                  None,
-    'unchanged':              None,
-    'update':                 None,
-    'long':                   None,
-    'root':                   str,
-    'no-backup':              None,
-    'pattern':                str,
-    'remember':               None,
-    }
-
-SHORT_OPTIONS = {
-    'F':                      'file', 
-    'h':                      'help',
-    'm':                      'message',
-    'r':                      'revision',
-    'v':                      'verbose',
-    'l':                      'long',
-    'q':                      'quiet',
-}
-
-
-def parse_args(argv):
+def parse_args(command, argv):
     """Parse command line.
     
     Arguments and options are parsed at this level before being passed
     down to specific command handlers.  This routine knows, from a
     lookup table, something about the available options, what optargs
     they take, and which commands will accept them.
-
-    >>> parse_args('--help'.split())
-    ([], {'help': True})
-    >>> parse_args('help -- --invalidcmd'.split())
-    (['help', '--invalidcmd'], {})
-    >>> parse_args('--version'.split())
-    ([], {'version': True})
-    >>> parse_args('status --all'.split())
-    (['status'], {'all': True})
-    >>> parse_args('commit --message=biter'.split())
-    (['commit'], {'message': u'biter'})
-    >>> parse_args('log -r 500'.split())
-    (['log'], {'revision': [<RevisionSpec_int 500>]})
-    >>> parse_args('log -r500..600'.split())
-    (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
-    >>> parse_args('log -vr500..600'.split())
-    (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
-    >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
-    (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})
     """
+    # TODO: chop up this beast; make it a method of the Command
     args = []
     opts = {}
 
+    cmd_options = command.options()
     argsover = False
     while argv:
         a = argv.pop(0)
-        if not argsover and a[0] == '-':
+        if argsover:
+            args.append(a)
+            continue
+        elif a == '--':
+            # We've received a standalone -- No more flags
+            argsover = True
+            continue
+        if a[0] == '-':
             # option names must not be unicode
             a = str(a)
             optarg = None
             if a[1] == '-':
-                if a == '--':
-                    # We've received a standalone -- No more flags
-                    argsover = True
-                    continue
                 mutter("  got option %r" % a)
                 if '=' in a:
                     optname, optarg = a[2:].split('=', 1)
                 else:
                     optname = a[2:]
-                if optname not in OPTIONS:
-                    raise BzrError('unknown long option %r' % a)
+                if optname not in cmd_options:
+                    raise BzrCommandError('unknown long option %r for command %s' 
+                            % (a, command.name))
             else:
                 shortopt = a[1:]
-                if shortopt in SHORT_OPTIONS:
+                if shortopt in Option.SHORT_OPTIONS:
                     # Multi-character options must have a space to delimit
                     # their value
-                    optname = SHORT_OPTIONS[shortopt]
+                    # ^^^ what does this mean? mbp 20051014
+                    optname = Option.SHORT_OPTIONS[shortopt].name
                 else:
                     # Single character short options, can be chained,
                     # and have their value appended to their name
                     shortopt = a[1:2]
-                    if shortopt not in SHORT_OPTIONS:
+                    if shortopt not in Option.SHORT_OPTIONS:
                         # We didn't find the multi-character name, and we
                         # didn't find the single char name
                         raise BzrError('unknown short option %r' % a)
-                    optname = SHORT_OPTIONS[shortopt]
+                    optname = Option.SHORT_OPTIONS[shortopt].name
 
                     if a[2:]:
                         # There are extra things on this option
                         # see if it is the value, or if it is another
                         # short option
-                        optargfn = OPTIONS[optname]
+                        optargfn = Option.OPTIONS[optname].type
                         if optargfn is None:
                             # This option does not take an argument, so the
                             # next entry is another short option, pack it back
@@ -465,7 +340,8 @@
                 # XXX: Do we ever want to support this, e.g. for -r?
                 raise BzrError('repeated option %r' % a)
                 
-            optargfn = OPTIONS[optname]
+            option_obj = cmd_options[optname]
+            optargfn = option_obj.type
             if optargfn:
                 if optarg == None:
                     if not argv:
@@ -479,12 +355,9 @@
                 opts[optname] = True
         else:
             args.append(a)
-
     return args, opts
 
 
-
-
 def _match_argform(cmd, takes_args, args):
     argdict = {}
 

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2005-10-17 10:50:39 +0000
+++ bzrlib/config.py	2005-10-17 11:56:54 +0000
@@ -381,3 +381,18 @@
     return realname, (username + '@' + socket.gethostname())
 
 
+def extract_email_address(e):
+    """Return just the address part of an email string.
+    
+    That is just the user@domain part, nothing else. 
+    This part is required to contain only ascii characters.
+    If it can't be extracted, raises an error.
+    
+    >>> extract_email_address('Jane Tester <jane@test.com>')
+    "jane@test.com"
+    """
+    m = re.search(r'[\w+.-]+@[\w+.-]+', e)
+    if not m:
+        raise BzrError("%r doesn't seem to contain "
+                       "a reasonable email address" % e)
+    return m.group(0)

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2005-10-12 07:28:41 +0000
+++ bzrlib/delta.py	2005-10-13 21:58:18 +0000
@@ -187,8 +187,8 @@
             # get them
             old_path = old_inv.id2path(file_id)
             new_path = new_inv.id2path(file_id)
-            old_ie._read_tree_state(old_tree)
-            new_ie._read_tree_state(new_tree)
+            old_ie._read_tree_state(old_path, old_tree)
+            new_ie._read_tree_state(new_path, new_tree)
             text_modified, meta_modified = new_ie.detect_changes(old_ie)
 
             # TODO: Can possibly avoid calculating path strings if the

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-10-17 08:15:09 +0000
+++ bzrlib/errors.py	2005-10-17 11:56:54 +0000
@@ -18,6 +18,17 @@
 __copyright__ = "Copyright (C) 2005 Canonical Ltd."
 __author__ = "Martin Pool <mbp@canonical.com>"
 
+# TODO: Change to a standard exception pattern: 
+#
+# - docstring of exceptions is a template for formatting the exception
+#   so the __str__ method can be defined only in the superclass
+# - the arguments to the exception are interpolated into this string
+#
+# when printing the exception we'd then require special handling only
+# for built-in exceptions with no decent __str__ method, such as 
+# ValueError and AssertionError.  See 
+# scott@canonical.com--2005/hct--devel--0.10 util/errors.py
+
 
 ######################################################################
 # exceptions 

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2005-10-11 03:19:29 +0000
+++ bzrlib/fetch.py	2005-10-11 05:36:02 +0000
@@ -111,11 +111,7 @@
             self.pb.clear()
 
     def _fetch_revisions(self, last_revision):
-        try:
-            self.last_revision = self._find_last_revision(last_revision)
-        except NoSuchRevision, e:
-            mutter('failed getting last revision: %s', e)
-            raise InstallFailed([last_revision])
+        self.last_revision = self._find_last_revision(last_revision)
         mutter('fetch up to rev {%s}', self.last_revision)
         if (self.last_revision is not None and 
             self.to_branch.has_revision(self.last_revision)):
@@ -134,13 +130,12 @@
 
         Returns the revision_id, or returns None if there's no history
         in the source branch."""
+        if last_revision:
+            return last_revision
         self.pb.update('get source history')
         from_history = self.from_branch.revision_history()
         self.pb.update('get destination history')
-        if last_revision:
-            self.from_branch.get_revision(last_revision)
-            return last_revision
-        elif from_history:
+        if from_history:
             return from_history[-1]
         else:
             return None                 # no history in the source branch

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-09-16 14:57:59 +0000
+++ bzrlib/help.py	2005-10-14 09:37:21 +0000
@@ -112,26 +112,28 @@
     outfile.write(doc)
     if doc[-1] != '\n':
         outfile.write('\n')
-    
-    help_on_options(cmd_object.takes_options, outfile=None)
-
-
-def help_on_options(options, outfile=None):
-    from bzrlib.commands import SHORT_OPTIONS
-    
+    help_on_command_options(cmd_object, outfile=None)
+
+
+def help_on_command_options(cmd, outfile=None):
+    from bzrlib.option import Option
+    options = cmd.options()
     if not options:
         return
-    
     if outfile == None:
         outfile = sys.stdout
-
     outfile.write('\noptions:\n')
-    for on in options:
-        l = '    --' + on
-        for shortname, longname in SHORT_OPTIONS.items():
-            if longname == on:
-                l += ', -' + shortname
-                break
+    for option_name, option in sorted(options.items()):
+        l = '    --' + option_name
+        if option.type is not None:
+            l += ' ' + option.argname.upper()
+        short_name = option.short_name()
+        if short_name:
+            assert len(short_name) == 1
+            l += ', -' + shortname
+        l += (30 - len(l)) * ' ' + option.help
+        # TODO: split help over multiple lines with correct indenting and 
+        # wrapping
         outfile.write(l + '\n')
 
 

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-10-14 02:17:36 +0000
+++ bzrlib/inventory.py	2005-10-16 05:15:58 +0000
@@ -133,12 +133,13 @@
         text_diff will be used for textual difference calculation.
         This is a template method, override _diff in child classes.
         """
-        self._read_tree_state(tree)
+        self._read_tree_state(tree.id2path(self.file_id), tree)
         if to_entry:
             # cannot diff from one kind to another - you must do a removal
             # and an addif they do not match.
             assert self.kind == to_entry.kind
-            to_entry._read_tree_state(to_tree)
+            to_entry._read_tree_state(to_tree.id2path(to_entry.file_id),
+                                      to_tree)
         self._diff(text_diff, from_label, tree, to_label, to_entry, to_tree,
                    output_to, reverse)
 
@@ -325,7 +326,7 @@
         text stored in the text store or weave.
         """
         mutter('new parents of %s are %r', path, previous_entries)
-        self._read_tree_state(work_tree)
+        self._read_tree_state(path, work_tree)
         if len(previous_entries) == 1:
             # cannot be unchanged unless there is only one parent file rev.
             parent_ie = previous_entries.values()[0]
@@ -391,7 +392,7 @@
             compatible = False
         return compatible
 
-    def _read_tree_state(self, work_tree):
+    def _read_tree_state(self, path, work_tree):
         """Populate fields in the inventory entry from the given tree.
         
         Note that this should be modified to be a noop on virtual trees
@@ -547,7 +548,7 @@
         if tree.is_executable(self.file_id):
             os.chmod(fullpath, 0755)
 
-    def _read_tree_state(self, work_tree):
+    def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
         self.text_sha1 = work_tree.get_file_sha1(self.file_id)
         self.executable = work_tree.is_executable(self.file_id)
@@ -657,7 +658,7 @@
         except OSError,e:
             raise BzrError("Failed to create symlink %r -> %r, error: %s" % (fullpath, self.symlink_target, e))
 
-    def _read_tree_state(self, work_tree):
+    def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
         self.symlink_target = work_tree.get_symlink_target(self.file_id)
 

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2005-10-10 03:24:15 +0000
+++ bzrlib/log.py	2005-10-13 16:16:50 +0000
@@ -54,6 +54,7 @@
 from bzrlib.tree import EmptyTree
 from bzrlib.delta import compare_trees
 from bzrlib.trace import mutter
+import re
 
 
 def find_touching_revisions(branch, file_id):
@@ -227,24 +228,25 @@
                 continue
 
         lf.show(revno, rev, delta)
-        if revno == 1:
-            excludes = set()
-        else:
-            # revno is 1 based, so -2 to get back 1 less.
-            excludes = set(branch.get_ancestry(revision_history[revno - 2]))
-        pending = list(rev.parent_ids)
-        while pending:
-            rev_id = pending.pop()
-            if rev_id in excludes:
-                continue
-            # prevent showing merged revs twice if they multi-path.
-            excludes.add(rev_id)
-            try:
-                rev = branch.get_revision(rev_id)
-            except errors.NoSuchRevision:
-                continue
-            pending.extend(rev.parent_ids)
-            lf.show_merge(rev)
+        if hasattr(lf, 'show_merge'):
+            if revno == 1:
+                excludes = set()
+            else:
+                # revno is 1 based, so -2 to get back 1 less.
+                excludes = set(branch.get_ancestry(revision_history[revno - 2]))
+            pending = list(rev.parent_ids)
+            while pending:
+                rev_id = pending.pop()
+                if rev_id in excludes:
+                    continue
+                # prevent showing merged revs twice if they multi-path.
+                excludes.add(rev_id)
+                try:
+                    rev = branch.get_revision(rev_id)
+                except errors.NoSuchRevision:
+                    continue
+                pending.extend(rev.parent_ids)
+                lf.show_merge(rev)
 
 
 def deltas_for_log_dummy(branch, which_revs):
@@ -340,9 +342,6 @@
     def show(self, revno, rev, delta):
         raise NotImplementedError('not implemented in abstract base')
 
-    def show_merge(self, rev):
-        pass
-
     
 class LongLogFormatter(LogFormatter):
     def show(self, revno, rev, delta):
@@ -408,7 +407,8 @@
         from bzrlib.osutils import format_date
 
         to_file = self.to_file
-
+        date_str = format_date(rev.timestamp, rev.timezone or 0,
+                            self.show_timezone)
         print >>to_file, "%5d %s\t%s" % (revno, rev.committer,
                 format_date(rev.timestamp, rev.timezone or 0,
                             self.show_timezone))
@@ -426,18 +426,51 @@
             delta.show(to_file, self.show_ids)
         print
 
-
+class LineLogFormatter(LogFormatter):
+    def truncate(self, str, max_len):
+        if len(str) <= max_len:
+            return str
+        return str[:max_len-3]+'...'
+
+    def date_string(self, rev):
+        from bzrlib.osutils import format_date
+        return format_date(rev.timestamp, rev.timezone or 0, 
+                           self.show_timezone, date_fmt="%Y-%m-%d",
+                           show_offset=False)
+
+    def message(self, rev):
+        if not rev.message:
+            return '(no message)'
+        else:
+            return rev.message
+
+    def short_committer(self, rev):
+        return re.sub('<.*@.*>', '', rev.committer).strip(' ')
+    
+    def show(self, revno, rev, delta):
+        print >> self.to_file, self.log_string(rev, 79) 
+
+    def log_string(self, rev, max_chars):
+        out = [self.truncate(self.short_committer(rev), 20)]
+        out.append(self.date_string(rev))
+        out.append(self.message(rev).replace('\n', ' '))
+        return self.truncate(" ".join(out).rstrip('\n'), max_chars)
+
+def line_log(rev, max_chars):
+    lf = LineLogFormatter(None)
+    return lf.log_string(rev, max_chars)
 
 FORMATTERS = {'long': LongLogFormatter,
               'short': ShortLogFormatter,
+              'line': LineLogFormatter,
               }
 
 
 def log_formatter(name, *args, **kwargs):
     """Construct a formatter from arguments.
 
-    name -- Name of the formatter to construct; currently 'long' and
-        'short' are supported.
+    name -- Name of the formatter to construct; currently 'long', 'short' and
+        'line' are supported.
     """
     from bzrlib.errors import BzrCommandError
     try:

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-10-16 22:31:25 +0000
+++ bzrlib/merge.py	2005-10-17 05:18:20 +0000
@@ -124,7 +124,7 @@
 
     def new_contents_conflict(self, filename, other_contents):
         """Conflicting contents for newly added file."""
-        self.copy(other_contents, filename + ".OTHER")
+        other.contents.apply(filename + ".OTHER")
         self.conflict("Conflict in newly added file %s" % filename)
     
 
@@ -153,12 +153,11 @@
 
     def abs_this_path(self, file_id):
         """Return the absolute path for a file_id in the this tree."""
-        relpath = self.this_tree.id2path(file_id)
-        return self.this_tree.tree.abspath(relpath)
+        return self.this_tree.id2abspath(file_id)
 
     def add_missing_parents(self, file_id, tree):
         """If some of the parents for file_id are missing, add them."""
-        entry = tree.tree.inventory[file_id]
+        entry = tree.inventory[file_id]
         if entry.parent_id not in self.this_tree:
             return self.create_all_missing(entry.parent_id, tree)
         else:
@@ -166,7 +165,7 @@
 
     def create_all_missing(self, file_id, tree):
         """Add contents for a file_id and all its parents to a tree."""
-        entry = tree.tree.inventory[file_id]
+        entry = tree.inventory[file_id]
         if entry.parent_id is not None and entry.parent_id not in self.this_tree:
             abspath = self.create_all_missing(entry.parent_id, tree)
         else:
@@ -178,8 +177,8 @@
 
     def create(self, file_id, path, tree, reverse=False):
         """Uses tree data to create a filesystem object for the file_id"""
-        from merge_core import get_id_contents
-        get_id_contents(file_id, tree)(path, self, reverse)
+        from changeset import get_contents
+        get_contents(tree, file_id)(path, self, reverse)
 
     def missing_for_merge(self, file_id, other_path):
         """The file_id doesn't exist in THIS, but does in OTHER and BASE"""
@@ -190,11 +189,15 @@
         self.create(file_id, stem+".OTHER", self.other_tree)
         self.create(file_id, stem+".BASE", self.base_tree)
 
+    def threeway_contents_conflict(filename, this_contents, base_contents,
+                                   other_contents):
+        self.conflict("Three-way conflict merging %s" % filename)
+
     def finalize(self):
         if not self.ignore_zero:
             note("%d conflicts encountered.\n" % self.conflicts)
             
-def get_tree(treespec, temp_root, label, local_branch=None):
+def get_tree(treespec, local_branch=None):
     location, revno = treespec
     branch = Branch.open_containing(location)
     if revno is None:
@@ -203,10 +206,9 @@
         revision = branch.last_revision()
     else:
         revision = branch.get_rev_id(revno)
-    return branch, get_revid_tree(branch, revision, temp_root, label,
-                                  local_branch)
+    return branch, get_revid_tree(branch, revision, local_branch)
 
-def get_revid_tree(branch, revision, temp_root, label, local_branch):
+def get_revid_tree(branch, revision, local_branch):
     if revision is None:
         base_tree = branch.working_tree()
     else:
@@ -215,91 +217,13 @@
             base_tree = local_branch.revision_tree(revision)
         else:
             base_tree = branch.revision_tree(revision)
-    temp_path = os.path.join(temp_root, label)
-    os.mkdir(temp_path)
-    return MergeAdapterTree(base_tree, temp_path)
+    return base_tree
 
 
 def file_exists(tree, file_id):
     return tree.has_filename(tree.id2path(file_id))
     
 
-class MergeAdapterTree(object):
-    """MergeAdapterTree adapts a normal tree for merge_inner to use.
-
-    The interface the merge_inner needs is nearly but not quite
-    the same as that of bzrlib.tree with the exception of readonly_path.
-    """
-    
-    def __init__(self, tree, tempdir):
-        object.__init__(self)
-        if hasattr(tree, "basedir"):
-            self.root = tree.basedir
-        else:
-            self.root = None
-        self.tree = tree
-        self.tempdir = tempdir
-        os.mkdir(os.path.join(self.tempdir, "texts"))
-        os.mkdir(os.path.join(self.tempdir, "symlinks"))
-        self.cached = {}
-
-    def __iter__(self):
-        return self.tree.__iter__()
-
-    def __contains__(self, file_id):
-        return file_id in self.tree
-
-    def get_file(self, file_id):
-        return self.tree.get_file(file_id)
-
-    def get_file_sha1(self, id):
-        return self.tree.get_file_sha1(id)
-
-    def is_executable(self, id):
-        return self.tree.is_executable(id)
-
-    def id2path(self, file_id):
-        return self.tree.id2path(file_id)
-
-    def has_id(self, file_id):
-        return self.tree.has_id(file_id)
-
-    def has_or_had_id(self, file_id):
-        if file_id == self.tree.inventory.root.file_id:
-            return True
-        return self.tree.inventory.has_id(file_id)
-
-    def has_or_had_id(self, file_id):
-        if file_id == self.tree.inventory.root.file_id:
-            return True
-        return self.tree.inventory.has_id(file_id)
-
-    def readonly_path(self, id):
-        if id not in self.tree:
-            return None
-        if self.root is not None:
-            return self.tree.abspath(self.tree.id2path(id))
-        else:
-            kind = self.tree.inventory[id].kind
-            if kind in ("directory", "root_directory"):
-                return self.tempdir
-            if not self.cached.has_key(id):
-                if kind == "file":
-                    path = os.path.join(self.tempdir, "texts", id)
-                    outfile = file(path, "wb")
-                    outfile.write(self.tree.get_file(id).read())
-                    assert(bzrlib.osutils.lexists(path))
-                    if self.tree.is_executable(id):
-                        os.chmod(path, 0755)
-                else:
-                    assert kind == "symlink"
-                    path = os.path.join(self.tempdir, "symlinks", id)
-                    target = self.tree.get_symlink_target(id)
-                    os.symlink(target, path)
-                self.cached[id] = path
-            return self.cached[id]
-
-
 def build_working_dir(to_dir):
     """Build a working directory in an empty directory.
 
@@ -349,8 +273,7 @@
                                     this_branch.basis_tree(), False)
             if changes.has_changed():
                 raise BzrCommandError("Working tree has uncommitted changes.")
-        other_branch, other_tree = get_tree(other_revision, tempdir, "other",
-                                            this_branch)
+        other_branch, other_tree = get_tree(other_revision, this_branch)
         if other_revision[1] == -1:
             other_rev_id = other_branch.last_revision()
             if other_rev_id is None:
@@ -370,11 +293,10 @@
                                               this_branch)
             except NoCommonAncestor:
                 raise UnrelatedBranches()
-            base_tree = get_revid_tree(this_branch, base_rev_id, tempdir, 
-                                       "base", None)
+            base_tree = get_revid_tree(this_branch, base_rev_id, None)
             base_is_ancestor = True
         else:
-            base_branch, base_tree = get_tree(base_revision, tempdir, "base")
+            base_branch, base_tree = get_tree(base_revision)
             if base_revision[1] == -1:
                 base_rev_id = base_branch.last_revision()
             elif base_revision[1] is None:
@@ -392,7 +314,7 @@
             for fname in file_list:
                 path = this_tree.relpath(fname)
                 found_id = False
-                for tree in (this_tree, base_tree.tree, other_tree.tree):
+                for tree in (this_tree, base_tree, other_tree):
                     file_id = tree.inventory.path2id(path)
                     if file_id is not None:
                         interesting_ids.add(file_id)
@@ -428,10 +350,10 @@
             contents_change = BackupBeforeChange(contents_change)
         return contents_change
 
-    this_tree = get_tree((this_branch.base, None), tempdir, "this")[1]
+    this_tree = get_tree((this_branch.base, None))[1]
 
     def get_inventory(tree):
-        return tree.tree.inventory
+        return tree.inventory
 
     inv_changes = merge_flex(this_tree, base_tree, other_tree,
                              generate_changeset, get_inventory,
@@ -450,7 +372,8 @@
             path = path[2:]
         adjust_ids.append((path, id))
     if len(adjust_ids) > 0:
-        this_branch.set_inventory(regen_inventory(this_branch, this_tree.root,
+        this_branch.set_inventory(regen_inventory(this_branch, 
+                                                  this_tree.basedir,
                                                   adjust_ids))
 
 

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-10-12 07:28:41 +0000
+++ bzrlib/merge_core.py	2005-10-14 04:53:26 +0000
@@ -5,6 +5,7 @@
 from bzrlib.osutils import backup_file, rename
 from bzrlib.merge3 import Merge3
 import bzrlib
+from changeset import get_contents
 
 class ApplyMerge3:
     """Contents-change wrapper around merge3.Merge3"""
@@ -12,7 +13,13 @@
         self.file_id = file_id
         self.base = base
         self.other = other
- 
+
+    def is_creation(self):
+        return False
+
+    def is_deletion(self):
+        return False
+
     def __eq__(self, other):
         if not isinstance(other, ApplyMerge3):
             return False
@@ -35,11 +42,6 @@
                 raise Exception("%s not in tree" % self.file_id)
                 return ()
             return tree.get_file(self.file_id).readlines()
-        ### garh. 
-        other_entry = other.tree.inventory[self.file_id]
-        if other_entry.kind == 'symlink':
-            self.apply_symlink(other_entry, base, other, filename)
-            return
         base_lines = get_lines(base)
         other_lines = get_lines(other)
         m3 = Merge3(base_lines, file(filename, "rb").readlines(), other_lines)
@@ -63,42 +65,18 @@
             conflict_handler.merge_conflict(new_file, filename, base_lines,
                                             other_lines)
 
-    def apply_symlink(self, other_entry, base, other, filename):
-        if self.file_id in base:
-            base_entry = base.tree.inventory[self.file_id]
-            base_entry._read_tree_state(base.tree)
-        else:
-            base_entry = None
-        other_entry._read_tree_state(other.tree)
-        if not base_entry or other_entry.detect_changes(base_entry):
-            other_change = True
-        else:
-            other_change = False
-        this_link = os.readlink(filename)
-        if not base_entry or base_entry.symlink_target != this_link:
-            this_change = True
-        else:
-            this_change = False
-        if this_change and not other_change:
-            pass
-        elif not this_change and other_change:
-            os.unlink(filename)
-            os.symlink(other_entry.symlink_target, filename)
-        elif this_change and other_change:
-            # conflict
-            os.unlink(filename)
-            os.symlink(other_entry.symlink_target, filename + '.OTHER')
-            os.symlink(this_link, filename + '.THIS')
-            if base_entry is not None:
-                os.symlink(other_entry.symlink_target, filename + '.BASE')
-            note("merge3 conflict in '%s'.\n", filename)
-
 
 class BackupBeforeChange:
     """Contents-change wrapper to back up file first"""
     def __init__(self, contents_change):
         self.contents_change = contents_change
- 
+
+    def is_creation(self):
+        return self.contents_change.is_creation()
+
+    def is_deletion(self):
+        return self.contents_change.is_deletion()
+
     def __eq__(self, other):
         if not isinstance(other, BackupBeforeChange):
             return False
@@ -129,8 +107,8 @@
     cset = changeset_function(base, other, interesting_ids)
     new_cset = make_merge_changeset(cset, this, base, other, 
                                     conflict_handler, merge_factory)
-    result = apply_changeset(new_cset, invert_invent(this.tree.inventory),
-                             this.root, conflict_handler, False)
+    result = apply_changeset(new_cset, invert_invent(this.inventory),
+                             this.basedir, conflict_handler, False)
     conflict_handler.finalize()
     return result
 
@@ -139,12 +117,6 @@
 def make_merge_changeset(cset, this, base, other, 
                          conflict_handler, merge_factory):
     new_cset = changeset.Changeset()
-    def get_this_contents(id):
-        path = this.readonly_path(id)
-        if os.path.isdir(path):
-            return changeset.dir_create
-        else:
-            return changeset.FileCreate(file(path, "rb").read())
 
     for entry in cset.entries.itervalues():
         if entry.is_boring():
@@ -188,7 +160,7 @@
         assert hasattr(tree, "__contains__"), "%s" % tree
         if not tree.has_or_had_id(file_id):
             return (None, None, "")
-        entry = tree.tree.inventory[file_id]
+        entry = tree.inventory[file_id]
         my_dir = tree.id2path(entry.parent_id)
         if my_dir is None:
             my_dir = ""
@@ -233,27 +205,15 @@
     return new_entry
 
 
-def get_contents(entry, tree):
-    return get_id_contents(entry.id, tree)
-
-def get_id_contents(file_id, tree):
-    """Get a contents change element suitable for use with ReplaceContents
-    """
-    tree_entry = tree.tree.inventory[file_id]
-    if tree_entry.kind == "file":
-        return changeset.FileCreate(tree.get_file(file_id).read())
-    elif tree_entry.kind == "symlink":
-        return changeset.SymlinkCreate(tree.get_symlink_target(file_id))
-    else:
-        assert tree_entry.kind in ("root_directory", "directory")
-        return changeset.dir_create
-
 def make_merged_contents(entry, this, base, other, conflict_handler,
                          merge_factory):
     contents = entry.contents_change
     if contents is None:
         return None
-    this_path = this.readonly_path(entry.id)
+    if entry.id in this:
+        this_path = this.id2abspath(entry.id)
+    else:
+        this_path = None
     def make_merge():
         if this_path is None:
             return conflict_handler.missing_for_merge(entry.id, 
@@ -264,7 +224,7 @@
         if contents.old_contents is None and contents.new_contents is None:
             return None
         if contents.new_contents is None:
-            this_contents = get_contents(entry, this)
+            this_contents = get_contents(this, entry.id)
             if this_path is not None and bzrlib.osutils.lexists(this_path):
                 if this_contents != contents.old_contents:
                     return conflict_handler.rem_contents_conflict(this_path, 
@@ -276,53 +236,57 @@
             if this_path is None or not bzrlib.osutils.lexists(this_path):
                 return contents
             else:
-                this_contents = get_contents(entry, this)
+                this_contents = get_contents(this, entry.id)
                 if this_contents == contents.new_contents:
                     return None
                 else:
-                    other_path = other.readonly_path(entry.id)    
                     conflict_handler.new_contents_conflict(this_path, 
-                                                           other_path)
-        elif (isinstance(contents.old_contents, changeset.FileCreate)
-              and isinstance(contents.new_contents, changeset.FileCreate)):
-            return make_merge()
-        elif (isinstance(contents.old_contents, changeset.SymlinkCreate)
-              and isinstance(contents.new_contents, changeset.SymlinkCreate)):
+                                                           other_contents)
+        elif isinstance(contents.old_contents, changeset.TreeFileCreate) and \
+            isinstance(contents.new_contents, changeset.TreeFileCreate):
             return make_merge()
         else:
-            raise Exception("Unhandled merge scenario")
+            this_contents = get_contents(this, entry.id)
+            if this_contents == contents.old_contents:
+                return contents
+            elif this_contents == contents.new_contents:
+                return None
+            elif contents.old_contents == contents.new_contents:
+                return None
+            else:
+                conflict_handler.threeway_contents_conflict(this_path, 
+                    this_contents, contents.old_contents,
+                    contents.new_contents)
+                
 
 def make_merged_metadata(entry, base, other):
     metadata = entry.metadata_change
     if metadata is None:
         return None
-    if isinstance(metadata, changeset.ChangeExecFlag):
-        if metadata.new_exec_flag is None:
-            return None
-        elif metadata.old_exec_flag is None:
-            return metadata
-        else:
-            base_path = base.readonly_path(entry.id)
-            other_path = other.readonly_path(entry.id)    
-            return ExecFlagMerge(base_path, other_path)
+    assert isinstance(metadata, changeset.ChangeExecFlag)
+    if metadata.new_exec_flag is None:
+        return None
+    elif metadata.old_exec_flag is None:
+        return metadata
+    else:
+        return ExecFlagMerge(base, other, entry.id)
     
 
 class ExecFlagMerge(object):
-    def __init__(self, base_path, other_path):
-        self.base_path = base_path
-        self.other_path = other_path
+    def __init__(self, base_tree, other_tree, file_id):
+        self.base_tree = base_tree
+        self.other_tree = other_tree
+        self.file_id = file_id
 
     def apply(self, filename, conflict_handler, reverse=False):
         if not reverse:
-            base = self.base_path
-            other = self.other_path
+            base = self.base_tree
+            other = self.other_tree
         else:
-            base = self.other_path
-            other = self.base_path
-        base_mode = os.stat(base).st_mode
-        base_exec_flag = bool(base_mode & 0111)
-        other_mode = os.stat(other).st_mode
-        other_exec_flag = bool(other_mode & 0111)
+            base = self.other_tree
+            other = self.base_tree
+        base_exec_flag = base.is_executable(self.file_id)
+        other_exec_flag = other.is_executable(self.file_id)
         this_mode = os.stat(filename).st_mode
         this_exec_flag = bool(this_mode & 0111)
         if (base_exec_flag != other_exec_flag and

=== added file 'bzrlib/option.py'
--- bzrlib/option.py	1970-01-01 00:00:00 +0000
+++ bzrlib/option.py	2005-10-17 04:16:40 +0000
@@ -0,0 +1,197 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import re
+
+import bzrlib.commands
+from bzrlib.trace import warning, mutter
+from bzrlib.revisionspec import RevisionSpec
+
+
+def _parse_revision_str(revstr):
+    """This handles a revision string -> revno.
+
+    This always returns a list.  The list will have one element for
+    each revision specifier supplied.
+
+    >>> _parse_revision_str('234')
+    [<RevisionSpec_int 234>]
+    >>> _parse_revision_str('234..567')
+    [<RevisionSpec_int 234>, <RevisionSpec_int 567>]
+    >>> _parse_revision_str('..')
+    [<RevisionSpec None>, <RevisionSpec None>]
+    >>> _parse_revision_str('..234')
+    [<RevisionSpec None>, <RevisionSpec_int 234>]
+    >>> _parse_revision_str('234..')
+    [<RevisionSpec_int 234>, <RevisionSpec None>]
+    >>> _parse_revision_str('234..456..789') # Maybe this should be an error
+    [<RevisionSpec_int 234>, <RevisionSpec_int 456>, <RevisionSpec_int 789>]
+    >>> _parse_revision_str('234....789') #Error ?
+    [<RevisionSpec_int 234>, <RevisionSpec None>, <RevisionSpec_int 789>]
+    >>> _parse_revision_str('revid:test@other.com-234234')
+    [<RevisionSpec_revid revid:test@other.com-234234>]
+    >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
+    >>> _parse_revision_str('revid:test@other.com-234234..23')
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_int 23>]
+    >>> _parse_revision_str('date:2005-04-12')
+    [<RevisionSpec_date date:2005-04-12>]
+    >>> _parse_revision_str('date:2005-04-12 12:24:33')
+    [<RevisionSpec_date date:2005-04-12 12:24:33>]
+    >>> _parse_revision_str('date:2005-04-12T12:24:33')
+    [<RevisionSpec_date date:2005-04-12T12:24:33>]
+    >>> _parse_revision_str('date:2005-04-12,12:24:33')
+    [<RevisionSpec_date date:2005-04-12,12:24:33>]
+    >>> _parse_revision_str('-5..23')
+    [<RevisionSpec_int -5>, <RevisionSpec_int 23>]
+    >>> _parse_revision_str('-5')
+    [<RevisionSpec_int -5>]
+    >>> _parse_revision_str('123a')
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: '123a'
+    >>> _parse_revision_str('abc')
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: 'abc'
+    >>> _parse_revision_str('branch:../branch2')
+    [<RevisionSpec_branch branch:../branch2>]
+    """
+    # TODO: Maybe move this into revisionspec.py
+    old_format_re = re.compile('\d*:\d*')
+    m = old_format_re.match(revstr)
+    revs = []
+    if m:
+        warning('Colon separator for revision numbers is deprecated.'
+                ' Use .. instead')
+        for rev in revstr.split(':'):
+            if rev:
+                revs.append(RevisionSpec(int(rev)))
+            else:
+                revs.append(RevisionSpec(None))
+    else:
+        next_prefix = None
+        for x in revstr.split('..'):
+            if not x:
+                revs.append(RevisionSpec(None))
+            elif x[-1] == ':':
+                # looks like a namespace:.. has happened
+                next_prefix = x + '..'
+            else:
+                if next_prefix is not None:
+                    x = next_prefix + x
+                revs.append(RevisionSpec(x))
+                next_prefix = None
+        if next_prefix is not None:
+            revs.append(RevisionSpec(next_prefix))
+    return revs
+
+
+def _parse_merge_type(typestring):
+    return bzrlib.commands.get_merge_type(typestring)
+
+
+class Option(object):
+    """Description of a command line option"""
+    # TODO: Some way to show in help a description of the option argument
+
+    OPTIONS = {}
+    SHORT_OPTIONS = {}
+
+    def __init__(self, name, help='', type=None, argname=None):
+        """Make a new command option.
+
+        name -- regular name of the command, used in the double-dash
+            form and also as the parameter to the command's run() 
+            method.
+
+        help -- help message displayed in command help
+
+        type -- function called to parse the option argument, or 
+            None (default) if this option doesn't take an argument.
+
+        argname -- name of option argument, if any
+        """
+        # TODO: perhaps a subclass that automatically does 
+        # --option, --no-option for reversable booleans
+        self.name = name
+        self.help = help
+        self.type = type
+        if type is None:
+            assert argname is None
+        elif argname is None:
+            argname = 'ARG'
+        self.argname = argname
+
+    def short_name(self):
+        """Return the single character option for this command, if any.
+
+        Short options are globally registered.
+        """
+        return Option.SHORT_OPTIONS.get(self.name)
+
+
+def _global_option(name, **kwargs):
+    """Register o as a global option."""
+    Option.OPTIONS[name] = Option(name, **kwargs)
+
+_global_option('all')
+_global_option('basis', type=str)
+_global_option('diff-options', type=str)
+_global_option('help',
+               help='show help message')
+_global_option('file', type=unicode)
+_global_option('force')
+_global_option('format', type=unicode)
+_global_option('forward')
+_global_option('message', type=unicode)
+_global_option('no-recurse')
+_global_option('profile',
+               help='show performance profiling information')
+_global_option('revision', type=_parse_revision_str)
+_global_option('short')
+_global_option('show-ids', 
+               help='show internal object ids')
+_global_option('timezone', 
+               type=str,
+               help='display timezone as local, original, or utc')
+_global_option('verbose',
+               help='display more information')
+_global_option('version')
+_global_option('email')
+_global_option('update')
+_global_option('long')
+_global_option('root', type=str)
+_global_option('no-backup')
+_global_option('merge-type', type=_parse_merge_type)
+_global_option('pattern', type=str)
+_global_option('quiet')
+_global_option('remember')
+
+
+def _global_short(short_name, long_name):
+    assert short_name not in Option.SHORT_OPTIONS
+    Option.SHORT_OPTIONS[short_name] = Option.OPTIONS[long_name]
+    
+
+Option.SHORT_OPTIONS['F'] = Option.OPTIONS['file']
+Option.SHORT_OPTIONS['h'] = Option.OPTIONS['help']
+Option.SHORT_OPTIONS['m'] = Option.OPTIONS['message']
+Option.SHORT_OPTIONS['r'] = Option.OPTIONS['revision']
+Option.SHORT_OPTIONS['v'] = Option.OPTIONS['verbose']
+Option.SHORT_OPTIONS['l'] = Option.OPTIONS['long']
+Option.SHORT_OPTIONS['q'] = Option.OPTIONS['quiet']

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-10-16 22:04:54 +0000
+++ bzrlib/osutils.py	2005-10-17 05:18:20 +0000
@@ -288,7 +288,8 @@
         return -time.timezone
 
     
-def format_date(t, offset=0, timezone='original'):
+def format_date(t, offset=0, timezone='original', date_fmt=None, 
+                show_offset=True):
     ## TODO: Perhaps a global option to use either universal or local time?
     ## Or perhaps just let people set $TZ?
     assert isinstance(t, float)
@@ -306,9 +307,13 @@
     else:
         raise BzrError("unsupported timezone format %r" % timezone,
                        ['options are "utc", "original", "local"'])
-
-    return (time.strftime("%a %Y-%m-%d %H:%M:%S", tt)
-            + ' %+03d%02d' % (offset / 3600, (offset / 60) % 60))
+    if date_fmt is None:
+        date_fmt = "%a %Y-%m-%d %H:%M:%S"
+    if show_offset:
+        offset_str = ' %+03d%02d' % (offset / 3600, (offset / 60) % 60)
+    else:
+        offset_str = ''
+    return (time.strftime(date_fmt, tt) +  offset_str)
 
 
 def compact_date(when):

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-10-17 04:27:17 +0000
+++ bzrlib/selftest/__init__.py	2005-10-17 11:56:54 +0000
@@ -33,7 +33,7 @@
 import bzrlib.osutils as osutils
 from bzrlib.selftest import TestUtil
 from bzrlib.selftest.TestUtil import TestLoader, TestSuite
-
+from bzrlib.selftest.treeshape import build_tree_contents
 
 MODULES_TO_TEST = []
 MODULES_TO_DOCTEST = []
@@ -121,10 +121,13 @@
 
 
 class TextTestRunner(unittest.TextTestRunner):
+    stop_on_failure = False
 
     def _makeResult(self):
         result = _MyResult(self.stream, self.descriptions, self.verbosity)
-        return EarlyStoppingTestResultAdapter(result)
+        if self.stop_on_failure:
+            result = EarlyStoppingTestResultAdapter(result)
+        return result
 
 
 def iter_suite_tests(suite):
@@ -196,6 +199,12 @@
             return
         raise AssertionError("texts not equal:\n" + 
                              self._ndiff_strings(a, b))      
+
+    def assertContainsRe(self, haystack, needle_re):
+        """Assert that a contains something matching a regular expression."""
+        if not re.search(needle_re, haystack):
+            raise AssertionError('pattern "%s" not found in "%s"'
+                    % (needle_re, haystack))
         
     def _enable_file_logging(self):
         fileno, name = tempfile.mkstemp(suffix='.log', prefix='testbzr')
@@ -433,6 +442,9 @@
                 print >>f, "contents of", name
                 f.close()
 
+    def build_tree_contents(self, shape):
+        bzrlib.selftest.build_tree_contents(shape)
+
     def failUnlessExists(self, path):
         """Fail unless path, which may be abs or relative, exists."""
         self.failUnless(osutils.lexists(path))
@@ -444,7 +456,7 @@
         logging.info('an info message')
         warning('something looks dodgy...')
         logging.debug('hello, test is running')
-        ##assert 0
+        ## assert 0
 
 
 def filter_suite_by_re(suite, pattern):
@@ -456,7 +468,8 @@
     return result
 
 
-def run_suite(suite, name='test', verbose=False, pattern=".*"):
+def run_suite(suite, name='test', verbose=False, pattern=".*",
+              stop_on_failure=False):
     TestCaseInTempDir._TEST_NAME = name
     if verbose:
         verbosity = 2
@@ -465,6 +478,7 @@
     runner = TextTestRunner(stream=sys.stdout,
                             descriptions=0,
                             verbosity=verbosity)
+    runner.stop_on_failure=stop_on_failure
     if pattern != '.*':
         suite = filter_suite_by_re(suite, pattern)
     result = runner.run(suite)
@@ -479,9 +493,10 @@
     return result.wasSuccessful()
 
 
-def selftest(verbose=False, pattern=".*"):
+def selftest(verbose=False, pattern=".*", stop_on_failure=True):
     """Run the whole test suite under the enhanced runner"""
-    return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern)
+    return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern,
+                     stop_on_failure=stop_on_failure)
 
 
 def test_suite():
@@ -532,6 +547,7 @@
                    'bzrlib.selftest.testtestament',
                    'bzrlib.selftest.testannotate',
                    'bzrlib.selftest.testrevprops',
+                   'bzrlib.selftest.testoptions',
                    ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-10-17 10:50:39 +0000
+++ bzrlib/selftest/blackbox.py	2005-10-17 11:56:54 +0000
@@ -26,9 +26,9 @@
 
 from cStringIO import StringIO
 import os
+import re
 import shutil
 import sys
-import os
 
 from bzrlib.branch import Branch
 from bzrlib.clone import copy_branch
@@ -401,8 +401,6 @@
         os.chdir('../b')
         self.runbzr('commit -m blah3 --unchanged')
         self.runbzr('pull ../a', retcode=1)
-        print "DECIDE IF PULL CAN CONVERGE, blackbox.py"
-        return
         os.chdir('../a')
         self.runbzr('merge ../b')
         self.runbzr('commit -m blah4 --unchanged')
@@ -671,7 +669,7 @@
         f.close()
 
         f = file('msg.tmp', 'wt')
-        f.write('this is my new commit\n')
+        f.write('this is my new commit\nand it has multiple lines, for fun')
         f.close()
 
         runbzr('commit -F msg.tmp')
@@ -685,11 +683,16 @@
         runbzr('log -v --forward')
         runbzr('log -m', retcode=1)
         log_out = capture('log -m commit')
-        assert "this is my new commit" in log_out
+        assert "this is my new commit\n  and" in log_out
         assert "rename nested" not in log_out
         assert 'revision-id' not in log_out
         assert 'revision-id' in capture('log --show-ids -m commit')
 
+        log_out = capture('log --line')
+        for line in log_out.splitlines():
+            assert len(line) <= 79, len(line)
+        assert "this is my new commit and" in log_out
+
 
         progress("file with spaces in name")
         mkdir('sub directory')
@@ -817,3 +820,5 @@
         self.run_bzr('branch', url, 'to')
         branch = Branch.open('to')
         self.assertEqual(1, len(branch.revision_history()))
+
+

=== modified file 'bzrlib/selftest/test_merge_core.py'
--- bzrlib/selftest/test_merge_core.py	2005-10-12 07:28:41 +0000
+++ bzrlib/selftest/test_merge_core.py	2005-10-14 04:53:26 +0000
@@ -2,17 +2,19 @@
 import shutil
 import tempfile
 import unittest
+import stat
 
 from bzrlib.selftest import TestCaseInTempDir, TestCase
 from bzrlib.branch import ScratchBranch, Branch
 from bzrlib.errors import NotBranchError, NotVersionedError
 from bzrlib.inventory import RootEntry
 import bzrlib.inventory as inventory
-from bzrlib.osutils import file_kind, rename
+from bzrlib.osutils import file_kind, rename, sha_file
 from bzrlib import changeset
 from bzrlib.merge_core import (ApplyMerge3, make_merge_changeset,
                                BackupBeforeChange, ExecFlagMerge)
 from bzrlib.changeset import Inventory, apply_changeset, invert_dict
+from bzrlib.changeset import get_contents, ReplaceContents
 
 
 class FalseTree(object):
@@ -27,14 +29,14 @@
         return entry
         
     def make_inventory_entry(self, file_id):
-        path = self._realtree.inventory.get(file_id)
+        path = self._realtree.inventory_dict.get(file_id)
         if path is None:
             return None
         if path == "":
             return RootEntry(file_id)
         dir, name = os.path.split(path)
         kind = file_kind(self._realtree.abs_path(path))
-        for parent_id, path in self._realtree.inventory.iteritems():
+        for parent_id, path in self._realtree.inventory_dict.iteritems():
             if path == dir:
                 break
         if path != dir:
@@ -53,11 +55,11 @@
     def __init__(self, dir):
         self.dir = dir;
         os.mkdir(dir)
-        self.inventory = {'0': ""}
-        self.tree = FalseTree(self)
+        self.inventory_dict = {'0': ""}
+        self.inventory = FalseTree(self)
     
     def child_path(self, parent, name):
-        return os.path.join(self.inventory[parent], name)
+        return os.path.join(self.inventory_dict[parent], name)
 
     def add_file(self, id, parent, name, contents, mode):
         path = self.child_path(parent, name)
@@ -65,11 +67,18 @@
         assert not os.path.exists(full_path)
         file(full_path, "wb").write(contents)
         os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
+        self.inventory_dict[id] = path
+
+    def add_symlink(self, id, parent, name, target):
+        path = self.child_path(parent, name)
+        full_path = self.abs_path(path)
+        assert not os.path.exists(full_path)
+        os.symlink(target, full_path)
+        self.inventory_dict[id] = path
 
     def remove_file(self, id):
         os.unlink(self.full_path(id))
-        del self.inventory[id]
+        del self.inventory_dict[id]
 
     def add_dir(self, id, parent, name, mode):
         path = self.child_path(parent, name)
@@ -77,14 +86,14 @@
         assert not os.path.exists(full_path)
         os.mkdir(self.abs_path(path))
         os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
+        self.inventory_dict[id] = path
 
     def abs_path(self, path):
         return os.path.join(self.dir, path)
 
     def full_path(self, id):
         try:
-            tree_path = self.inventory[id]
+            tree_path = self.inventory_dict[id]
         except KeyError:
             return None
         return self.abs_path(tree_path)
@@ -93,23 +102,38 @@
         return self.full_path(id)
 
     def __contains__(self, file_id):
-        return file_id in self.inventory
+        return file_id in self.inventory_dict
 
     def has_or_had_id(self, file_id):
         return file_id in self
 
     def get_file(self, file_id):
-        path = self.readonly_path(file_id)
+        path = self.full_path(file_id)
         return file(path, "rb")
 
     def id2path(self, file_id):
-        return self.inventory[file_id]
+        return self.inventory_dict[file_id]
+
+    def id2abspath(self, id):
+        return self.full_path(id)
 
     def change_path(self, id, path):
-        old_path = os.path.join(self.dir, self.inventory[id])
+        old_path = os.path.join(self.dir, self.inventory_dict[id])
         rename(old_path, self.abs_path(path))
-        self.inventory[id] = path
-
+        self.inventory_dict[id] = path
+
+    def is_executable(self, file_id):
+        mode = os.lstat(self.full_path(file_id)).st_mode
+        return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
+
+    def kind(self, file_id):
+        return file_kind(self.full_path(file_id))
+
+    def get_symlink_target(self, file_id):
+        return os.readlink(self.full_path(file_id))
+
+    def get_file_sha1(self, file_id):
+        return sha_file(file(self.full_path(file_id), "rb"))
 
 class MergeBuilder(object):
     def __init__(self):
@@ -134,6 +158,13 @@
         path = self.get_cset_path(parent, name)
         self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
 
+    def add_symlink(self, id, parent, name, contents):
+        self.base.add_symlink(id, parent, name, contents)
+        self.this.add_symlink(id, parent, name, contents)
+        self.other.add_symlink(id, parent, name, contents)
+        path = self.get_cset_path(parent, name)
+        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
+
     def remove_file(self, id, base=False, this=False, other=False):
         for option, tree in ((base, self.base), (this, self.this), 
                              (other, self.other)):
@@ -145,7 +176,7 @@
                     change = changeset.ReplaceContents(None, None)
                     self.cset.entries[id].contents_change = change
                     def create_file(tree):
-                        return changeset.FileCreate(tree.get_file(id).read())
+                        return changeset.TreeFileCreate(tree, id)
                     if not other:
                         change.new_contents = create_file(self.other)
                     if not base:
@@ -206,8 +237,23 @@
         if base is not None or other is not None:
             old_contents = file(self.base.full_path(id)).read()
             new_contents = file(self.other.full_path(id)).read()
-            contents = changeset.ReplaceFileContents(old_contents, 
-                                                     new_contents)
+            contents = changeset.ReplaceFileContents(self.base, self.other, id)
+            self.cset.entries[id].contents_change = contents
+
+    def change_target(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_target_tree(id, self.base, base)
+
+        if this is not None:
+            self.change_target_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_target_tree(id, self.other, other)
+
+        if base is not None or other is not None:
+            old_contents = get_contents(self.base, id)
+            new_contents = get_contents(self.other, id)
+            contents = ReplaceContents(old_contents, new_contents)
             self.cset.entries[id].contents_change = contents
 
     def change_perms(self, id, base=None, this=None, other=None):
@@ -221,8 +267,8 @@
             self.change_perms_tree(id, self.other, other)
 
         if base is not None or other is not None:
-            old_exec = bool(os.stat(self.base.full_path(id)).st_mode & 0111)
-            new_exec = bool(os.stat(self.other.full_path(id)).st_mode & 0111)
+            old_exec = self.base.is_executable(id)
+            new_exec = self.other.is_executable(id)
             metadata = changeset.ChangeExecFlag(old_exec, new_exec)
             self.cset.entries[id].metadata_change = metadata
 
@@ -240,6 +286,11 @@
         file(path, "w").write(contents)
         os.chmod(path, mode)
 
+    def change_target_tree(self, id, tree, target):
+        path = tree.full_path(id)
+        os.unlink(path)
+        os.symlink(target, path)
+
     def change_perms_tree(self, id, tree, mode):
         os.chmod(tree.full_path(id), mode)
 
@@ -289,11 +340,11 @@
 
     def apply_changeset(self, cset, conflict_handler=None, reverse=False):
         inventory_change = changeset.apply_changeset(cset,
-                                                     self.this.inventory,
+                                                     self.this.inventory_dict,
                                                      self.this.dir,
                                                      conflict_handler, reverse)
-        self.this.inventory =  self.apply_inv_change(inventory_change, 
-                                                     self.this.inventory)
+        self.this.inventory_dict =  self.apply_inv_change(inventory_change, 
+                                                     self.this.inventory_dict)
 
     def cleanup(self):
         shutil.rmtree(self.dir)
@@ -317,7 +368,7 @@
         for tree in (builder.this, builder.other, builder.base):
             assert(tree.dir != builder.dir and 
                    tree.dir.startswith(builder.dir))
-            for path in tree.inventory.itervalues():
+            for path in tree.inventory_dict.itervalues():
                 fullpath = tree.abs_path(path)
                 assert(fullpath.startswith(tree.dir))
                 assert(not path.startswith(tree.dir))
@@ -340,11 +391,11 @@
         builder.add_file("4", "1", "file2", "hello2", 0644)
         builder.add_file("5", "1", "file3", "hello3", 0644)
         builder.change_parent("3", other="2")
-        assert(Inventory(builder.other.inventory).get_parent("3") == "2")
+        assert(Inventory(builder.other.inventory_dict).get_parent("3") == "2")
         builder.change_parent("4", this="2")
-        assert(Inventory(builder.this.inventory).get_parent("4") == "2")
+        assert(Inventory(builder.this.inventory_dict).get_parent("4") == "2")
         builder.change_parent("5", base="2")
-        assert(Inventory(builder.base.inventory).get_parent("5") == "2")
+        assert(Inventory(builder.base.inventory_dict).get_parent("5") == "2")
         cset = builder.merge_changeset(ApplyMerge3)
         for id in ("1", "2", "4", "5"):
             assert(cset.entries[id].is_boring())
@@ -427,20 +478,29 @@
                           cset)
         builder.cleanup()
 
+    def test_symlink_conflicts(self):
         builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", base=True)
-        self.assertRaises(changeset.NewContentsConflict,
-                          builder.merge_changeset, merge_factory)
+        builder.add_symlink("2", "0", "name2", "target1")
+        builder.change_target("2", other="target4", base="text3")
+        self.assertRaises(changeset.ThreewayContentsConflict,
+                          builder.merge_changeset, ApplyMerge3)
         builder.cleanup()
 
+    def test_symlink_merge(self):
         builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", this=True)
-        self.assertRaises(changeset.MissingForMerge, builder.merge_changeset, 
-                          merge_factory)
+        builder.add_symlink("1", "0", "name1", "target1")
+        builder.add_symlink("2", "0", "name2", "target1")
+        builder.add_symlink("3", "0", "name3", "target1")
+        builder.change_target("1", this="target2")
+        builder.change_target("2", base="target2")
+        builder.change_target("3", other="target2")
+        assert builder.cset.entries['2'].contents_change !=\
+            builder.cset.entries['3'].contents_change
+        cset = builder.merge_changeset(ApplyMerge3)
+        builder.apply_changeset(cset)
+        self.assertEqual(builder.this.get_symlink_target("1"), "target2")
+        self.assertEqual(builder.this.get_symlink_target("2"), "target1")
+        self.assertEqual(builder.this.get_symlink_target("3"), "target2")
         builder.cleanup()
 
     def test_perms_merge(self):
@@ -457,9 +517,9 @@
         assert(isinstance(cset.entries["2"].metadata_change, ExecFlagMerge))
         assert(cset.entries["3"].is_boring())
         builder.apply_changeset(cset)
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0644)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0644)
+        assert(os.lstat(builder.this.full_path("1")).st_mode &0100 == 0000)
+        assert(os.lstat(builder.this.full_path("2")).st_mode &0100 == 0100)
+        assert(os.lstat(builder.this.full_path("3")).st_mode &0100 == 0000)
         builder.cleanup();
 
 

=== modified file 'bzrlib/selftest/testannotate.py'
--- bzrlib/selftest/testannotate.py	2005-10-13 06:33:22 +0000
+++ bzrlib/selftest/testannotate.py	2005-10-17 02:46:47 +0000
@@ -39,6 +39,21 @@
 
 
 class TestAnnotate(TestCaseInTempDir):
+    def setUp(self):
+        super(TestAnnotate, self).setUp()
+        b = Branch.initialize('.')
+        self.build_tree_contents([('hello.txt', 'my helicopter\n')])
+        b.add(['hello.txt'])
+        b.commit('add hello', 
+                 committer='test@user')
+
     def test_help_annotate(self):
         """Annotate command exists"""
         out, err = self.run_bzr_captured(['--no-plugins', 'annotate', '--help'])
+
+    def test_annotate_cmd(self):
+        out, err = self.run_bzr_captured(['annotate', 'hello.txt'])
+        self.assertEquals(err, '')
+        self.assertEqualDiff(out, '''\
+    1 test@us | my helicopter
+''')

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-10-17 11:31:05 +0000
+++ bzrlib/selftest/testbranch.py	2005-10-17 11:56:54 +0000
@@ -25,6 +25,10 @@
 import bzrlib.transactions as transactions
 from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
 
+# TODO: Make a branch using basis branch, and check that it 
+# doesn't request any files that could have been avoided, by 
+# hooking into the Transport.
+
 class TestBranch(TestCaseInTempDir):
 
     def test_append_revisions(self):
@@ -83,9 +87,6 @@
         os.mkdir('c')
         br_c = copy_branch(br_a, 'c', basis_branch=br_b)
         self.assertEqual(br_a.revision_history(), br_c.revision_history())
-        ## # basis branches currently disabled for weave format
-        ## self.assertFalse(br_b.last_revision() in br_c.revision_history())
-        ## br_c.get_revision(br_b.last_revision())
 
     def test_copy_partial(self):
         """Copy only part of the history of a branch."""

=== added file 'bzrlib/selftest/testoptions.py'
--- bzrlib/selftest/testoptions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testoptions.py	2005-10-14 09:42:49 +0000
@@ -0,0 +1,52 @@
+# (C) 2005 Canonical
+
+from bzrlib.selftest import TestCase
+from bzrlib.commands import Command, parse_args
+from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
+
+
+class OptionTests(TestCase):
+    """Command-line option tests"""
+
+    def test_parse_args(self):
+        """Option parser"""
+        eq = self.assertEquals
+        eq(parse_args(cmd_commit(), ['--help']),
+           ([], {'help': True}))
+        eq(parse_args(cmd_status(), ['--all']),
+           ([], {'all': True}))
+        eq(parse_args(cmd_commit(), ['--message=biter']),
+           ([], {'message': 'biter'}))
+        ## eq(parse_args(cmd_log(),  '-r 500'.split()),
+        ##   ([], {'revision': RevisionSpec_int(500)}))
+
+    def test_no_more_opts(self):
+        """Terminated options"""
+        self.assertEquals(parse_args(cmd_commit(), ['--', '-file-with-dashes']),
+                          (['-file-with-dashes'], {}))
+
+    def test_option_help(self):
+        """Options have help strings."""
+        out, err = self.run_bzr_captured(['commit', '--help'])
+        self.assertContainsRe(out, r'--file.*file containing commit message')
+
+    def test_option_help_global(self):
+        """Global options have help strings."""
+        out, err = self.run_bzr_captured(['help', 'status'])
+        self.assertContainsRe(out, r'--show-ids.*show internal object')
+
+    def test_option_arg_help(self):
+        """Help message shows option arguments."""
+        out, err = self.run_bzr_captured(['help', 'commit'])
+        self.assertEquals(err, '')
+        self.assertContainsRe(out, r'--file[ =]MSGFILE')
+
+
+#     >>> parse_args('log -r 500'.split())
+#     (['log'], {'revision': [<RevisionSpec_int 500>]})
+#     >>> parse_args('log -r500..600'.split())
+#     (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+#     >>> parse_args('log -vr500..600'.split())
+#     (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+#     >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
+#     (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})

=== modified file 'bzrlib/selftest/testrevision.py'
--- bzrlib/selftest/testrevision.py	2005-10-02 23:17:00 +0000
+++ bzrlib/selftest/testrevision.py	2005-10-10 04:13:48 +0000
@@ -181,15 +181,13 @@
                                         self.br1.revision_history()), 
                          ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
                           'a@u-0-5'])
-        print ("testrevision.py 191 - intervene appears to return b..6 even"
-               "though it is not reachable!")
-#        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-6', 
-#                         self.br1.revision_history()), 
-#                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
-#                          'b@u-0-6'])
-#        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-5'), 
-#                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4', 
-#                          'b@u-0-5'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-6', 
+                         self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'b@u-0-6'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-5'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4', 
+                          'b@u-0-5'])
         self.assertEqual(self.intervene('b@u-0-3', 'b@u-0-6', 
                          self.br2.revision_history()), 
                          ['b@u-0-4', 'b@u-0-5', 'b@u-0-6'])

=== modified file 'bzrlib/selftest/teststatus.py'
--- bzrlib/selftest/teststatus.py	2005-10-03 01:53:47 +0000
+++ bzrlib/selftest/teststatus.py	2005-10-13 16:16:50 +0000
@@ -23,6 +23,12 @@
 
 from bzrlib.selftest import TestCaseInTempDir
 from bzrlib.revisionspec import RevisionSpec
+from bzrlib.merge import merge
+from cStringIO import StringIO
+from bzrlib.status import show_status
+from bzrlib.branch import Branch
+from os import mkdir
+from bzrlib.clone import copy_branch
 
 class BranchStatus(TestCaseInTempDir):
     
@@ -54,9 +60,6 @@
 
     def test_branch_status_revisions(self):
         """Tests branch status with revisions"""
-        from cStringIO import StringIO
-        from bzrlib.status import show_status
-        from bzrlib.branch import Branch
         
         b = Branch.initialize('.')
 
@@ -93,6 +96,31 @@
                            '  bye.c\n',
                            '  hello.c\n'])
 
+    def status_string(self, branch):
+        tof = StringIO()
+        show_status(branch, to_file=tof)
+        tof.seek(0)
+        return tof.getvalue()
+
+    def test_pending(self):
+        """Pending merges display works"""
+        mkdir("./branch")
+        b = Branch.initialize('./branch')
+        b.commit("Empty commit 1")
+        b_2 = copy_branch(b, './copy')
+        b.commit("Empty commit 2")
+        merge(["./branch", -1], [None, None], this_dir = './copy')
+        message = self.status_string(b_2)
+        assert (message.startswith("pending merges:\n")), message
+        assert (message.endswith("Empty commit 2\n")), message 
+        b_2.commit("merged")
+        b.commit("Empty commit 3 blah blah blah blah blah blah blah blah blah")
+        merge(["./branch", -1], [None, None], this_dir = './copy')
+        message = self.status_string(b_2)
+        assert (message.startswith("pending merges:\n")), message
+        assert ("Empty commit 3" in message), message
+        assert (message.endswith("...\n")), message 
+
     def test_branch_status_specific_files(self): 
         """Tests branch status with given specific files"""
         from cStringIO import StringIO

=== modified file 'bzrlib/selftest/testtestament.py'
--- bzrlib/selftest/testtestament.py	2005-10-13 03:23:22 +0000
+++ bzrlib/selftest/testtestament.py	2005-10-17 06:57:11 +0000
@@ -102,6 +102,19 @@
         t = Testament.from_revision(self.b, 'test@user-3')
         self.assertEqualDiff(t.as_text(), REV_3_TESTAMENT)
 
+    def test_testament_revprops(self):
+        """Testament to revision with extra properties"""
+        props = dict(flavor='sour cherry\ncream cheese',
+                     size='medium')
+        self.b.commit(message='revision with properties',
+                      timestamp=1129025493,
+                      timezone=36000,
+                      rev_id='test@user-3',
+                      committer='test@user',
+                      revprops=props)
+        t = Testament.from_revision(self.b, 'test@user-3')
+        self.assertEqualDiff(t.as_text(), REV_PROPS_TESTAMENT)
+                    
 
 REV_1_TESTAMENT = """\
 bazaar-ng testament version 1
@@ -146,6 +159,29 @@
 """ % sha(REV_2_TESTAMENT).hexdigest()
 
 
+REV_PROPS_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  revision with properties
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+properties:
+  flavor:
+    sour cherry
+    cream cheese
+  size:
+    medium
+"""
+
+
 REV_3_TESTAMENT = """\
 bazaar-ng testament version 1
 revision-id: test@user-3

=== modified file 'bzrlib/shellcomplete.py'
--- bzrlib/shellcomplete.py	2005-09-09 17:12:26 +0000
+++ bzrlib/shellcomplete.py	2005-10-14 08:37:15 +0000
@@ -28,17 +28,14 @@
         outfile.write(aname + '\n')
 
 
-def shellcomplete_on_option(options, outfile = None):
-    import commands
-    
+def shellcomplete_on_option(options, outfile=None):
+    from bzrlib.option import Option
     if not options:
         return
-    
     if outfile == None:
         outfile = sys.stdout
-
     for on in options:
-        for shortname, longname in commands.SHORT_OPTIONS.items():
+        for shortname, longname in Option.SHORT_OPTIONS.items():
             if longname == on:
                 l = '"(--' + on + ' -' + shortname + ')"{--' + on + ',-' + shortname + '}'
                 break

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-10-06 13:08:14 +0000
+++ bzrlib/status.py	2005-10-13 16:16:50 +0000
@@ -17,6 +17,8 @@
 import sys
 from bzrlib.osutils import is_inside_any
 from bzrlib.delta import compare_trees
+from bzrlib.log import line_log
+from bzrlib.errors import NoSuchRevision
 
 
 def show_status(branch, show_unchanged=False,
@@ -88,7 +90,12 @@
             if show_pending and len(branch.pending_merges()) > 0:
                 print >>to_file, 'pending merges:'
                 for merge in branch.pending_merges():
-                    print >> to_file, ' ', merge
+                    try:
+                        m_revision = branch.get_revision(merge)
+                        print >> to_file, ' ', line_log(m_revision, 77)
+                    except NoSuchRevision:
+                        print >> to_file, ' ', merge 
+                        
     finally:
         branch.unlock()
         

=== modified file 'bzrlib/testament.py'
--- bzrlib/testament.py	2005-10-14 04:23:13 +0000
+++ bzrlib/testament.py	2005-10-17 06:57:11 +0000
@@ -69,6 +69,7 @@
 # TODO: Perhaps these should just be different formats in which inventories/
 # revisions can be serialized.
 
+from copy import copy
 from cStringIO import StringIO
 import string
 from sha import sha
@@ -98,6 +99,7 @@
         t.message = rev.message
         t.parent_ids = rev.parent_ids[:]
         t.inventory = branch.get_inventory(revision_id)
+        t.revprops = copy(rev.properties)
         assert not contains_whitespace(t.revision_id)
         assert not contains_linebreaks(t.committer)
         return t
@@ -127,6 +129,7 @@
         a('inventory:\n')
         for path, ie in self.inventory.iter_entries():
             a(self._entry_to_line(path, ie))
+        r.extend(self._revprops_to_lines())
         if __debug__:
             for l in r:
                 assert isinstance(l, str), \
@@ -165,3 +168,17 @@
                 'sha1: %s\n'
                 % (self.revision_id, s.hexdigest()))
 
+    def _revprops_to_lines(self):
+        """Pack up revision properties."""
+        if not self.revprops:
+            return []
+        r = ['properties:\n']
+        for name, value in sorted(self.revprops.items()):
+            assert isinstance(name, str)
+            assert not contains_whitespace(name)
+            r.append('  %s:\n' % name)
+            for line in value.splitlines():
+                if not isinstance(line, str):
+                    line = line.encode('utf-8')
+                r.append('    %s\n' % line)
+        return r

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2005-10-07 06:13:14 +0000
+++ bzrlib/tree.py	2005-10-14 04:32:24 +0000
@@ -58,6 +58,11 @@
     def has_id(self, file_id):
         return self.inventory.has_id(file_id)
 
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
+            return True
+        return self.inventory.has_id(file_id)
+
     __contains__ = has_id
 
     def __iter__(self):
@@ -157,6 +162,9 @@
             return ie.text_sha1
 
     def is_executable(self, file_id):
+        ie = self._inventory[file_id]
+        if ie.kind != "file":
+            return None 
         return self._inventory[file_id].executable
 
     def has_filename(self, filename):
@@ -171,6 +179,8 @@
         ie = self._inventory[file_id]
         return ie.symlink_target;
 
+    def kind(self, file_id):
+        return self._inventory[file_id].kind
 
 class EmptyTree(Tree):
     def __init__(self):

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-10-16 22:09:09 +0000
+++ bzrlib/workingtree.py	2005-10-17 05:18:20 +0000
@@ -179,6 +179,10 @@
         path = inv.id2path(file_id)
         return bzrlib.osutils.lexists(self.abspath(path))
 
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
+            return True
+        return self.inventory.has_id(file_id)
 
     __contains__ = has_id
     
@@ -391,6 +395,9 @@
         else:
             return None
 
+    def kind(self, file_id):
+        return file_kind(self.id2abspath(file_id))
+
 CONFLICT_SUFFIXES = ('.THIS', '.BASE', '.OTHER')
 def get_conflicted_stem(path):
     for suffix in CONFLICT_SUFFIXES:

=== modified file 'contrib/zsh/_bzr'
--- contrib/zsh/_bzr	2005-05-09 04:30:00 +0000
+++ contrib/zsh/_bzr	2005-10-14 10:37:07 +0000
@@ -9,13 +9,12 @@
 # how to get zsh to reload this file when it changes, other than by
 # starting a new zsh.
 
-# This is not very useful at the moment because it only completes on
-# commands and breaks filename expansion for other arguments.
-
-
-_arguments -S "1::bzr command:($(bzr help commands | grep -v '^   '))"
-
-
-# Local variables:
-# mode: shell-script
-# End:
+local _bzr_subcommands expl curcontext="$curcontext"
+
+_bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
+
+if (( CURRENT == 2 )); then
+  _describe -t subcommand 'subcommand' _bzr_subcommands
+else
+# this part missing
+fi

